%
% It employs QDCT55 to quantize the DCTs patches with resolution N from a 2D signal
%
% QDCT55 allows designing the VQ using the IAF or the statistics, or to introduce some other quantizer
% previously design (contains APELES Y VQPERC, y QDCT21)
%
% Introduce 1 in the 'opc' variable to compute the PERCEPTUAL VQ.
% Then you need to give the the needed parameters to the function (See VQPERC.m).
%  * Quantization using a perceptual VQ needs:
%         señal,Ncuan,opc = 1,facfrec,ncontr,nolin,estab,
%           B,bitmax,fc,cc,crit,fijarmax,inclcsf,frecbaja.
%
% Introduce 2 in the 'opc' variable to compute the ESTATISTICAL VQ.
% Then you need to give the the needed parameters to the function (See APELES.M).
%  * Quantization using a statistical VQ needs:
%         señal,Ncuan,opc = 2,B,bitmax,fchest,fc,cc,crit,frecbaja
%         (fchest(fichero estadisticas con vari, ff, FF).)
%
% If using a precomputed VQ (opc = 0), it needs:
%         señal,Ncuan,opc = 0,niveles,fronteras,fc,cc,crit,frecbaja.
%
% We can choose between different DCT normalization:
%
%   Norm = 1 ......... Normalizing using the mean luminance in each patch.
%
%   Norm = 2 ......... Normalizing using the mean luminance in each patch and the neighbour patches.
%                      The number of neighbour patches are introduced as parameter.
%
%   Norm = 3 ......... Normalizing using the mean luminance in the whole image.
%
%   Norm = 4 ......... Normalizing using the variable 'valor' (valor*N^2).
%
%   Norm = 5 ......... Not normalization.
%
% USE: jpeg91_encoder(s,B,directorio,fichero)
%
% where:
%
% s         -> Image
% B         -> Quantization levels
% diretorio -> Folder to store the compressed file
% fichero   -> Name of the compressed file
%

function jpeg91_encoder(s,B,directorio,fichero)
Ncuan = 4;
opc = 1;
niveles = 0;
fronteras = 0;
facfrec = 1.1;

ncontr = 100;
nolin = 0;
estab = 0;
bitmax = 10;
met = 0;
fchest = 0;
fc = 32;
cc = 1;
crit = 1;
fijarmax = 0;
inclcsf = [1 1];
frecbaja = 0.001;
calcent = 1;

norm = [3 mean(mean(s))];
MM = 0.8;

tam = size(s);
tam = tam(1);
lados = [tam tam];
imq = zeros(tam,tam);
dctq = zeros(tam,tam);
codq = zeros(tam,tam);
lcuan = tam/(2^Ncuan);
posai = [tam tam]/2-round(tam/2);
posbd = [tam tam]/2+round(tam/2);
coorcuanai = floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
coorcuanbd = floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
if coorcuanai(1)<1
    coorcuanai(1)=1;
end
if coorcuanai(2)<1
    coorcuanai(2)=1;
end
if coorcuanbd(1)>(2^Ncuan)
    coorcuanbd(1)=2^Ncuan;
end
if coorcuanai(2)>(2^Ncuan)
    coorcuanbd(2)=2^Ncuan;
end
H=0;
Hi=0;
if (opc==2)
    [iaf,ciaf1D,niveles,fronteras]=apeles(B,bitmax,met,fchest);
    if inclcsf(1) == 1
        [ciaf1Dp,iafp,nuqfp,nivelesp,fronterasp] = vqperc(lcuan,facfrec,ncontr,nolin,estab,B,bitmax,fc,fijarmax,0);
        fronterasp = fronterasp*(lcuan^2)*128;

        tamfro = size(fronteras);
        for i = 1:tamfro(1)
            if niveles(i,1) ~= -1
                condic = (fronteras(i,:) <= (inclcsf(2)*fronterasp(i,1)));
                if (sum(condic) ~= 0)
                    v = find(condic<1);
                    l = length(v);
                    if l>1
                        front = [fronteras(i,v) maxi(fronteras(i,:))*ones(1,tamfro(2)-l)];
                        nivel = [0 niveles(i,v(2:l)) maxi(niveles(i,:))*ones(1,tamfro(2)-l+1)];
                        fronteras(i,:) = front;
                        niveles(i,:) = nivel;
                    else
                        fronteras(i,:) = maxi(fronteras(i,:))*ones(1,tamfro(2));
                        niveles(i,:) = [0 maxi(niveles(i,:))*ones(1,tamfro(2))];
                    end
                end
            else
                fronteras(i,:) = fronteras(i,:);
                niveles(i,:) = niveles(i,:);
            end
        end
    end
elseif (opc == 1)
    [ciaf1D,iaf,nuqf,niveles,fronteras] = vqperc(lcuan,facfrec,ncontr,nolin,estab,B,bitmax,fc,fijarmax,0);
end

dcti = dct2r(s,Ncuan);

[a,Lm] = contras2(dcti,Ncuan,norm);

pes = ones(lcuan,lcuan);
pes(1,1) = 0;

pes=[pes pes pes pes pes pes pes pes pes pes pes pes pes pes pes pes];
pes=[pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes];
a = pes.*a;

W = 1;

val_r = 100;

e = imdpcm(Lm,W,val_r);

Nc = round(0.9 * 40);
me = mini(e);
Me = maxi(e);
extr_err = [me Me];
QE = round((Nc -1) * (e - me) / (Me - me));

Lb = 16;
Li = 256;
codigazo = [];
for i = 1:Lb:Li
    for j = 1:Lb:Li
        cub = zeros(Lb,Lb);
        cub = a(i:(i+(Lb-1)),j:(j+(Lb-1)));
        fre = topolar3(fc,Lb);
        cond = fre > frecbaja;
        tfs = cub;
        tfs = tfs .* cond;
        tfc = zeros(Lb,Lb);
        cod = zeros(Lb,Lb);
        for i1 = 1:Lb,
            for j1 = 1:Lb,
                d = floor(sqrt((i1-1).^2+(j1-1).^2));
                [tfc(i1,j1),cod(i1,j1)] = asignbbb(abs(tfs(i1,j1)),niveles(d+1,:),fronteras(d+1,:),crit);

            end
        end
        c1 = (cub >= 0);
        c2 = (cub <  0);
        tfc = c1.*tfc - c2.*tfc;
        anticond = cond < 1;
        dctc = cc .* anticond .* cub + cond.*tfc;

        dctc = round(2^B * dctc / MM);
        bloque = zigzag(dctc);
        bloque = bloque(2:end);

        [codigo,ceros] = rle2(bloque);
        codigazo = [codigazo codigo ceros];
    end
end

ristra = [mean(mean(s)) extr_err(:)' round(QE(:))' B round(codigazo)];

cd(directorio);
save_code_jpeg91(ristra,[fichero '.bin']);
zip(fichero,[fichero '.bin']);

delete([fichero '.bin']);
