% Use QDCT65 to quantize the normalized DCTs of a subblock of a 2D signal
% (Currently 256 * 256 resolution 4)
%
% Differences from previous versions :
% - Use the proper normalization of contrasts ( contras2 rather than contrast )
% - Designs the normalization  Kernel taking proper care of the DC component.
% - Including in one function constants from naka8 and naka100
% ( With exponent 2 - naka8 - or 'optimized' exponent  - 0.98 - naka100 )
%
% Use one or the other parameters as a function of the value of the variable
% Exp:
% Exp = 2 -> values ​​naka8
% Exp = 1 - > values ​​naka100
% - Calculate the parameters once ( constantes_resp ) and then passes them to
% calculation of the response function ( respue5 ) .
%
% Difference of QDCT62 that allows you to not calculate the eigenvalues ​​of the original
% (if you're not interested in them, it saves time !)
%
% Represents QDCT62 signal in a domain with (approx.) statistically uncorrelated characteristics
% and uncorrelated perceptually (With perceptual metric identity).
% This is achieved through the divisive normalization: Teo & Heeger, Watson and Simoncelli.
% This transformation depends on its input signal ( state adaptation ) .
% It has been shown that this type of response obtains independent statistics ( Simoncelli99 ) and
% perceptually ( Malo99 ) coefficients , and therefore is a suitable domain for
% a scalar quantizer .
%
% Since the perceptual metric in this domain is the identity apply uniform quantizers
% and flat bit-allocation per coefficient.
%
% For each block the invertibility is checked [max( eig ( D_R * h )) ], and the inverse is computed
% by using the analytical expression
%
% It is normalized to local Michaelson contrast and response according NAKA8.m :
%
% Norm = [1 1] ................ Normalized by the average luminance
% of the subfield considered (local contrast) .
%
% Norm = [3 Lm ] ............... Normalized by the fixed value Lm (global contrast)
% the logical choice is to take Lm = mean ( mean ( Im) );
% ( Mean value in digital levels )
%
% Range to distribute the quantization levels in the domain response is given
% by the maximum diagonal gradient ( acting on contrasts, belonging to
% [ -1.1 ] ) .
%
% USE : RESP = malo06_encoder(s,B,exp,directorio,fichero,RESP)
%
% Where:
%
% S -> compressed image
% B - > Parameter setting of quantization levels
% Exp -> exponent nonlinearity. You can take the values ​​1 or 2
% directorio -> Place where the compressed file will be stored
% fichero -> name with which the compressed file will be stored
% RESP -> Optional parameter : nonlinear transform image
%	  In the first execution this parameter may not be available,
% 	  but in a loop search of a certain entropy or
% 	  distortion is calculated the first time and avoiding load
% 	  posterior calculation.

function RESP=malo06_encoder(s,B,exp,directorio,fichero,RESP)

if nargin<6
   RESP=[];
end

Lb = 16;
Li = 256;

M = 0.8;
cero = 1;

Norm = [3 mean(mean(s))];

Ncuan = 4;

tam = size(s);
tam = tam(1);
lados = [tam tam];
lcuan = tam/(2^Ncuan);
posai = [tam tam]/2-round(tam/2);
posbd = [tam tam]/2+round(tam/2);
coorcuanai = floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
coorcuanbd = floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
if coorcuanai(1) < 1
    coorcuanai(1) = 1;
end
if coorcuanai(2) < 1
    coorcuanai(2) = 1;
end
if coorcuanbd(1) > (2^Ncuan)
    coorcuanbd(1) = 2^Ncuan;
end
if coorcuanai(2) > (2^Ncuan)
    coorcuanbd(2) = 2^Ncuan;
end

pes = ones(lcuan,lcuan);
pes(1,1) = 0;

pes=[pes pes pes pes pes pes pes pes pes pes pes pes pes pes pes pes];
pes=[pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes;pes];

dcti = dct2r(s,Ncuan);

[a,Lm] = contras2(dcti,Ncuan,Norm);

a = pes.*a;

W = 1;

val_r = 100;

e = imdpcm(Lm,W,val_r);

Nc = round(0.9 * 40);

me = mini(e);
Me = maxi(e);

extr_err = [me Me];

QE = round((Nc -1) * (e - me) / (Me - me));

[h,kk1,kk2,gamm] = constrains_resp(exp,cero);

if nargin<6
fprintf('  Computing the non-linear response');
end

codigazo = [];
bloquecito = 1;
for i=1:Lb:Li

    if nargin<6
       fprintf('.');
    end

    for j=1:Lb:Li

        aa = a(i:(i+(Lb-1)),j:(j+(Lb-1)));

        if nargin<6

            [h_a,r,GR,alfa2d,beta2d] = respue5(aa,kk1,kk2,gamm,h,0);

            r = zigzag(r);
            r = r(2:end);
            RESP=[RESP r];

        else
            r=RESP(:,bloquecito);
        end

        azig = zigzag(aa);
        azigsc = azig(2:end);

        cod = round(2^B * r / M);
        rc = cod * M / (2^B);

      signo1 = sign(azigsc).*(rc~=0) + (rc==0);

        codigon = signo1 .* rc;
        codigonint = signo1 .* cod;

        [codigo,ceros] = rle2(codigonint);
        codigazo = [codigazo codigo ceros];

        bloquecito = bloquecito + 1;

    end
end

ristra = [mean(mean(s)) extr_err(:)' round(QE(:))' B round(codigazo)];

cd(directorio);
save_code_malo06(ristra,[fichero '.bin']);

zip(fichero,[fichero '.bin']);

delete([fichero '.bin']);
