% ------------------------------------------------- ------------------------
% Svr_dct_encoder block-DCT applied to an image and SVR
%
% USE: svr_dct_encoder (Im, bits, epsilon, H, listing, directory, file)
%
% Im 		-> Image to compress
% Bits 		-> Number of bits for quantization of the weights of the SVR
% Epsilon 	-> threshold used in the regression
% H 		-> kernel used in SVM
% perfil 	-> Profile for epsilon. It is a vector of the same
% 		size as the vector which is passed to the SVM and
% 		epsilon affect. This will be the difference for the
% 		CTE, CSF and RKi algorithms.
% diretorio 	-> Place where the compressed file will be stored
% fichero 	-> name with which the compressed file will be stored
% ------------------------------------------------- ------------------------

function svr_dct_encoder(Im,bits,epsilon,H,perfil,directorio,fichero)

MM = 0.8;
normaliz = [3 mean(mean(Im))];

Ncuan = 4;
Lb = 16;
Li = length(Im);

dcti = dct2r(Im,Ncuan);

[dct_contr,Lm] = contras2(dcti,Ncuan,normaliz);
DC = Lm*16;

W = 1;

val_r = 100;

e = imdpcm(Lm,W,val_r);

Nc = round(0.9 * 40);

me = mini(e);
Me = maxi(e);

extr_err = [me Me];

QE = round((Nc -1) * (e - me) / (Me - me));

R = 1:(Lb*Lb-1);
R = R(:);
N = length(R);
C = 40000;
D = ones(1,N);
D = D(:);

codigazo = [];

signosr = zeros(256,255);
betasc = zeros(256,255);
betascr = zeros(256,255);

fprintf('SVM learning the blocks of the local DCT');

bloquecito = 1;

for i=1:Lb:Li

    fprintf('.');

    for j=1:Lb:Li

        B = dct_contr(i:(i+(Lb-1)),j:(j+(Lb-1)));

        Bzig = zigzag(B);
        Bzig = Bzig(2:end);
        bitsig = getsign(Bzig,Lb);
        signosr(bloquecito,:) = bitsig;

        [nsv,beta,bias] = irwls_pd_svr_nobias(R,abs(Bzig),H,C,D,epsilon,perfil);

        betasc(bloquecito,:) = beta';

        bloquecito = bloquecito + 1;

    end
end

fprintf('\n');

NumNiv = 2^bits;

betascr = round(NumNiv * betasc / MM) * MM / NumNiv;

signos = zeros(size(signosr));
signos(logical(betascr ~= 0)) = signosr(logical(betascr ~= 0));

codigon = betascr.*(~signos) - betascr.*(signos);
codigonint = codigon * NumNiv / MM;

for i = 1:Lb^2
    [cod,ceros] = rle2(codigonint(i,:));
    codigazo = [codigazo cod ceros];
end

ristra = [mean(mean(Im)) extr_err(:)' round(QE(:))' 10*bits round(codigazo)];

cd(directorio);
save_code_svr_dct(ristra,[fichero '.bin']);

zip(fichero,[fichero '.bin']);

delete([fichero '.bin']);
