% SVR_NL DCTs 'retransformadas' of a 2D signal subblock
% (Currently 256 * 256 resolution 4)
%
% Difference of QDCT63_2 in allowing the introduction of a mask
% (16 * 16 ) selecting certain coefficients so that,
% increased sequentially, you can simulate the progressive decoding
% (sending bits... : see progressive_coding.m ).
% This mask is introduced into the variable 'progres'.
% If progres = ones ( 16,16 ), the result is the same as using qdct63_2
%
% Difference of QDCT62 that allows you to not calculate the eigenvalues ​​of the original
% (if you're not interested, it saves time!)
%
% Represents QDCT62 signal in a domain with (approx.) statistically uncorrelated characteristics
% and uncorrelated perceptually (With perceptual metric identity).
%
% This is achieved through the divisive normalization: Teo & Heeger, Watson and Simoncelli.
% This transformation depends on its input signal ( state adaptation ).
%
% It has been shown that this type of response obtains independent statistics ( Simoncelli99 ) and
% perceptually ( Malo99 ) coefficients, and therefore is a suitable domain for
% a scalar quantizer.
%
% Since the perceptual metric in this domain is the identity apply uniform quantizers
% and flat bit-allocation per coefficient.
%
% For each block the invertibility is checked [max( eig ( D_R * h )) ], and the inverse is computed
% by using the analytical expression
%
% It is normalized to local Michaelson contrast and response according NAKA8.m:
%
% Norm = [1 1] ................ Normalized by the average luminance
% of the subfield considered (local contrast) .
%
% Norm = [3 Lm] ............... Normalized by the fixed value Lm (global contrast)
% the logical choice is to take Lm = mean ( mean ( Im) );
% ( Mean value in digital levels )
%
% Range to distribute the quantization levels in the domain response is given
% by the maximum diagonal gradient (acting on contrasts, belonging to
% [ -1.1 ]).
%
% USE: RESP=svr_nl_encoder(Im,bits,epsilon,K,exponente,directorio,fichero,RESP)
%
% Im        -> Image to compress
% bits      -> Numerb of bits for the SVR weights quantization
% epsilon   -> Regression threshold
% K         -> SVR Kernel
% directorio -> Place where the compressed file will be stored
% fichero -> name with which the compressed file will be stored
% RESP -> Optional parameter : nonlinear transform image
%	  In the first execution this parameter may not be available,
% 	  but in a loop search of a certain entropy or
% 	  distortion is calculated the first time and avoiding load
% 	  posterior calculation.

function RESP=svr_nl_encoder(Im,bits,epsilon,K,exponente,directorio,fichero,RESP)

if nargin<8
   RESP=[];
end

MM = 0.8;
normaliz = [3 mean(mean(Im))];
cero = 1;

[h,alpha,beta,gamm] = constrains_resp(exponente,cero);

Ncuan=4;
tam=size(Im);
tam=tam(1);
lados=[tam tam];
lcuan=tam/(2^Ncuan);

posai=[tam tam]/2-round(tam/2);
posbd=[tam tam]/2+round(tam/2);
coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
if coorcuanai(1)<1
    coorcuanai(1)=1;
end
if coorcuanai(2)<1
    coorcuanai(2)=1;
end
if coorcuanbd(1)>(2^Ncuan)
    coorcuanbd(1)=2^Ncuan;
end
if coorcuanai(2)>(2^Ncuan)
    coorcuanbd(2)=2^Ncuan;
end

dcti=dct2r(Im,Ncuan);

[dct_contr,Lm] = contras2(dcti,Ncuan,normaliz);
DC = Lm*16;

W=1;

val_r = 100;

e = imdpcm(Lm,W,val_r);

N = round(0.9 * 40);

me = mini(e);
Me = maxi(e);

extr_err = [me Me];

QE = round((N -1) * (e - me) / (Me - me));

R=(1:255);R=R(:);

C=40000;
perfil_C=ones(255,1);

perfil_e=ones(255,1);

Lb = lcuan;
Li = tam;

Results=[];

codigazo = [];

signosr = zeros(256,255);
betasc =  zeros(256,255);
betascr = zeros(256,255);

if nargin<8
   fprintf('  Computing the non-linear response \n');
   fprintf('  SVM learning the blocks of the local non-linear domain');
else
   fprintf('  SVM learning the blocks of the local non-linear domain');
end

bloquecito = 1;
for i=1:Lb:Li

    fprintf('.');

    for j=1:Lb:Li

        aa = dct_contr(i:(i+(Lb-1)),j:(j+(Lb-1)));

        if nargin<8
           [h_a,r,GR,alfa2d,beta2d] = respue5(aa,alpha,beta,gamm,h,0);

           r=zigzag(r);

           RESP=[RESP r];
        else
           r=RESP(:,bloquecito);
        end

        [nsv,pesosSVM,bias] = irwls_pd_svr_nobias(R,r(2:256),K,C,perfil_C,epsilon,perfil_e);

        azig = zigzag(aa);
        azigsc = azig(2:end);

        bitsig = getsign(azigsc,Lb);

        signosr(bloquecito,:) = bitsig;
        betasc(bloquecito,:) = pesosSVM';

        bloquecito = bloquecito + 1;

    end
end
fprintf('\n');

NumNiv = 2^bits;

betascr = round( NumNiv * betasc / MM) * MM / NumNiv;

rc = [];
for fila = 1:Li
    rc = [rc; (K*betascr(fila,:)')'];
end

signos = zeros(size(signosr));
signos(logical(abs(rc)>0))=signosr(logical(abs(rc)>0));

codigon = betascr.*(~signos) - betascr.*(signos);

codigonint = codigon * NumNiv / MM;

for i=1:Lb^2

        [cod,ceros] = rle2(codigonint(i,:));

        codigazo = [codigazo cod ceros];

end

ristra = [mean(mean(Im)) extr_err(:)' round(QE(:))' bits round(codigazo)];

cd(directorio);
save_code_svr_nl(ristra,[fichero '.bin']);

zip(fichero,[fichero '.bin']);

delete([fichero '.bin']);

