
% MALO99 quantizes 16*16 DCT blocks of 256*256 images using adaptive bit allocation
% and non-uniform quantizers for each coefficient based on a (point-wise) non-linear
% human vision model (Malo95, Malo99, Malo00).
%
% In this implementation the allocated bits are controlled by the
% control parameter, CP.
% Increasing CP reduces the quantization step sizes, increasing the entropy,
% and reducing the distortion.
%
% The algorithm starts by using a CP value provided by the user, uCP.
% This CP value is iteratively modified till the desired entropy is achieved.
%
% USE: [Results] = malo99_2(Im,parametro,B,variable)
%
% INPUT:
%
% Im        -> Image
% parametro -> Distortion parameter value
% B         -> Control parameter
% variable  -> Text string to choose the distortion parameter
%
% OUTPUT:
%
% Results   -> Struct with the information a bout results
%              to the diostortion parameter:
%              struct('uCP',B,'Image',ImR,'Entropy',H)
%              Results.SNR, Results.MSE, Results.SSIM, Results.MPE_lineal,
%              Results.MPE_no_lineal, Results.MPE_no_linea_JMLR
%
% REFERENCES:
%
%    Malo et al., Electr. Lett., 1995. (Preliminary version).
%    Malo et al., Electr. Lett., 1999. (Restriction of Maximum Perceptual Error).
%    Malo et al., Im. Vis. Comp, 2000. (General MPE framework and links with JPEG).
%

function   [Results] = malo99_ucp_distortion(Im,param,B_inicial,variable);

Im=double(Im);

Ncuan=4;
opc=1;
niveles=0;
fronteras=0;
facfreq=1;
ncontr=100;
nolin=1;
estab=0;
Bitmax=10;
metod=0;
fich_est=0;
fc=32;
cc=1;
crit=1;
fijar_max=0;
inccsf=[1 1];
f_baja=0.001;
cal_entr=1;
norma=[1 0];

Results=[];

contador = 0;

for parametro = param

    disp(['Adjustment of uCP to reach desired ' variable ':   ' num2str(parametro)]);

    switch(variable)
        case 'SNR', parametro_actual = 1; umbral = 0.05;
        case 'MSE', parametro_actual = 1; umbral = 0.5;
        case 'SSIM', parametro_actual = 0; umbral = 0.001;
        case 'MPE lineal', parametro_actual = 100; umbral = 0.01;
        case 'MPE no lineal', parametro_actual = 100; umbral = 0.01;
        case 'MPE no lineal JMLR', parametro_actual = 100; umbral = 0.1;
    end

    iter = 0;

    B = 0;

    porcentaje = 1;

    if(isequal(variable,'SSIM'))
        sentido = 1;
    else
        sentido = 0;
    end

    no_results = 0;

    l_min = 0;
    l_max = 100000;

    while(abs(parametro - parametro_actual) > umbral) & (iter < 100)

        iter = iter + 1;

        if(iter ~= 1 & iter ~= 2)
            switch(variable)
                case 'SNR',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP_SSIM(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MSE',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'SSIM',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP_SSIM(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MPE lineal',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MPE no lineal',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
                case 'MPE no lineal JMLR',
                    [B, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro, parametro_actual, B, l_min, l_max, porcentaje, sentido);
            end
        end

        [imq,dctq,cod,ciaf1D,iaf,nuqf,niveles,fronteras,H,Hi]=qdct57(Im,Ncuan,opc,niveles,fronteras,facfreq,ncontr,nolin,estab,B,Bitmax,metod,fich_est,fc,cc,crit,fijar_max,inccsf,f_baja,cal_entr,norma);
        ImR = imq;

        switch(variable)
            case 'SNR',
                [mse,SNRi] = emse2(Im,ImR);
                fprintf('   It. =
                parametro_actual = SNRi;
            case 'MSE',
                [mse,SNRi] = emse2(Im,ImR);
                fprintf('   It. =
                parametro_actual = mse;
            case 'SSIM',
                esesim = ssim_index(Im,ImR);
                fprintf('   It. =
                parametro_actual = esesim;
            case 'MPE lineal',
                [MSE,d_l,dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(Im,ImR,2,[3 mean(mean(Im))],2,2);
                mpelin = d_l;
                fprintf('   It. =
                parametro_actual = mpelin;
            case 'MPE no lineal',
                [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],2,1,2,2);
                mpenonlin= d;
                fprintf('   It. =
                parametro_actual = mpenonlin;
            case 'MPE no lineal JMLR',
                [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],1,0.5,2,2);
                mpenonlinJMLR = d;
                fprintf('   It. =
                parametro_actual = mpenonlinJMLR;
        end

        if(~isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual > 0) & (B == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP para esta distorsion');
            break;

        elseif(isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual < 0) & (B == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP para esta distorsion');
            break;

        elseif((iter == 1) & (abs(parametro - parametro_actual) <= umbral) & (B == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP para esta distorsion');
            break;

        elseif(iter == 1)
            B = B_inicial;
        end
    end

    if(no_results == 0)

        contador = contador + 1;

        H = Hi/(size(Im,1) * size(Im,2));

        Results(contador).uCP = B;
        Results(contador).Image = ImR;
        Results(contador).Entropy = H;

        switch(variable)
            case 'SNR',
                Results(contador).SNR = SNRi;
            case 'MSE',
                Results(contador).MSE = mse;
            case 'SSIM',
                Results(contador).SSIM = esesim;
            case 'MPE lineal',
                Results(contador).MPE_lineal = mpelin;
            case 'MPE no lineal',
                Results(contador).MPE_no_lineal = mpenonlin;
            case 'MPE no lineal JMLR',
                Results(contador).MPE_no_lineal_JMLR = mpenonlinJMLR;
        end
    end
end
