% ------------------------------------------------- --------------------------------------
% SVR_dct calculates the rate-distortion curve for a picture (using
% constant tube until the cutoff frequency fc -en - cpd ).
%
% From a given distortion parameter (SNR , MSE , SSIM , MPE) it adjusts
% the value of epsilon SVR.
% Using this epsilon (and profile depending on the used algorithm),
% DCT blocks are trained (with a fixed C, and predefined profile and a 2D kernel
% variance 1).
% Finally , it calculates the entropy for the result.
%
% USAGE: [ Results] = svr_dct_2 (Im , bits, parameter, epsilon , H Profile, sepsigno , variable)
%
% Im 		-> Image
% Bits 		-> N number of bits for quantization of SVM weights
% Parameter 	-> value pair parameter of distortion selected
% Epsilon 	-> Initial value of epsilon
% H 		-> Kernel used in SVR
% Profile 	-> Profile epsilon for each frequency
% Sepsigno 	-> Flag to account for the entropy of signs ( '1 ') or
% 		no ( '0 ')
% Variable 	-> Text string to select the parameter of distortion
% 		in question
%
% Results 	-> Vector of struct containing information about the results
% 		for the given parameter of distortion :
% 		Struct ( 'bits ' bits, 'epsilon' , epsilon , 'NumNiv', NumNiv,
%		'Image', ImR , 'Entropy', entropiat, 'Entropiasig', entropsn, 'entropiapes', entropcn)
% 		Results.SNR , Results.MSE , Results.SSIM , Results.MPE_lineal ,
% 		Results.MPE_no_lineal , Results.MPE_no_linea_JMLR

function Results = svr_dct_epsilon_bits_distortion(Im,param,epsilon_inicial,H,perfil,sepsigno,variable)

Lb = 16;
Li = length(Im);

R = 1:(Lb*Lb-1);
R = R(:);
N = length(R);

C = 40000;
D = ones(1,N);
D = D(:);

Results = [];
ImR = rand(Li);

res = 0;

ini_bits = 2;

fin_bits = 8;

for parametro = param

    disp(['Adjustment of epsilon to reach desired ' variable ':   ' num2str(parametro)]);

    for bits = ini_bits:0.5:fin_bits

        switch(variable)
            case 'SNR', parametro_actual = 1; umbral = 0.05;
            case 'MSE', parametro_actual = 1; umbral = 0.5;
            case 'SSIM', parametro_actual = 0; umbral = 0.01;
            case 'MPE lineal', parametro_actual = 100; umbral = 0.01;
            case 'MPE no lineal', parametro_actual = 100; umbral = 0.01;
            case 'MPE no lineal JMLR', parametro_actual = 100; umbral = 0.1;
        end

        epsilon = 0;

        iter = 0;

        porcentaje = 1;

        if(isequal(variable,'SSIM'))
            sentido = 0;
        else
            sentido = 1;
        end

        no_results = 0;

        l_min = 0;
        l_max = 100000;

        while((abs(parametro - parametro_actual) > umbral) & (iter < 30))

            iter = iter + 1;
            signoscoefs = [];
            betasc = [];
            BtC = [];
            Coef11 = [];
            SVs = [];
            lSVs = [];

            if(iter ~= 1 & iter ~= 2)
                switch(variable)
                    case 'SNR',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon_SSIM(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MSE',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'SSIM',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon_SSIM(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MPE lineal',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MPE no lineal',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MPE no lineal JMLR',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                end
            end

            fprintf('Training all the blocks of the local DCT');

            for i = 1:Lb:Li
                fprintf('.');
                for j=1:Lb:Li

                    B = Im(i:(i+(Lb-1)),j:(j+(Lb-1)));
                    B = dct2(B);
                    Coef11 = [Coef11;B(1,1)];
                    B = zigzag(B);
                    B = B(2:end);
                    BtC = [BtC;B'];

                    bitsig = getsign(B,Lb);
                    signoscoefs = [signoscoefs; bitsig];

                    [nsv,beta,bias] = irwls_pd_svr_nobias(R,abs(B),H,C,D,epsilon,perfil);

                    betasc = [betasc;beta'];
                end
            end

            fprintf('\n');

            MM = max(max(abs(betasc)));
            MM=MM/4;
            NumNiv = 2^bits;
            betascr = round(NumNiv * betasc/MM)*MM/NumNiv;
            signos = zeros(size(signoscoefs));
            signos(logical(betascr~=0))=signoscoefs(logical(betascr~=0));

            z = 1;
            for i=1:Lb:Li
                for j=1:Lb:Li

                    y = H*betascr(z,:)';

                    y = obtainsign(y, signos, Lb,z);
                    A = dezigzag([Coef11(z);y]);
                    A = idct2(A);
                    ImR(i:(i+(Lb-1)),j:(j+(Lb-1))) = A;
                    z = z + 1;
                end
            end

            switch(variable)
                case 'SNR',
                    [mse,SNRi] = emse2(Im,ImR);
                    fprintf('   It. =
                    parametro_actual = SNRi;
                case 'MSE',
                    [mse,SNRi] = emse2(Im,ImR);
                    fprintf('   It. =
                    parametro_actual = mse;
                case 'SSIM',
                    esesim = ssim_index(Im,ImR);
                    fprintf('   It. =
                    parametro_actual = esesim;
                case 'MPE lineal',
                    [MSE,d_l,dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(Im,ImR,2,[3 mean(mean(Im))],2,2);
                    mpelin = d_l;
                    fprintf('   It. =
                    parametro_actual = mpelin;
                case 'MPE no lineal',
                    [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],2,1,2,2);
                    mpenonlin= d;
                    fprintf('   It. =
                    parametro_actual = mpenonlin;
                case 'MPE no lineal JMLR',
                    [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],1,0.5,2,2);
                    mpenonlinJMLR = d;
                    fprintf('   It. =
                    parametro_actual = mpenonlinJMLR;
            end

            if(~isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual < 0) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits');
                break;

            elseif(isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual > 0) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits');
                break;

            elseif((iter == 1) & (abs(parametro - parametro_actual) <= umbral) & (epsilon == 0))

                epsilon = epsilon_inicial;

                parametro_actual = 0;
            elseif(iter == 1)
                epsilon = epsilon_inicial;
            end

        end

        if(no_results == 0)

            entropc = [];
            entrops = [];

            codigon = betascr.*(~signos) - betascr.*(signos);
            for i=1:Lb^2

                if sepsigno==0
                    codigo = betascr(i,:).*(~signos(i,:)) - betascr(i,:).*(signos(i,:));
                    [rlePesos,repCeros] = rle2(codigo);
                    entropc = [entropc length(rlePesos)*entropiv(rlePesos)+length(repCeros)*entropiv(repCeros)];
                    entrops = 0;
                else

                    [rlePesos,repCeros] = rle2(betascr(i,:));
                    entropc = [entropc length(rlePesos)*entropiv(rlePesos)+length(repCeros)*entropiv(repCeros)];

                    [rlePesoss,repCeross] = rle2(signos(i,:));
                    entrops = [entrops length(rlePesoss)*entropiv(rlePesoss)+length(repCeross)*entropiv(repCeross)];
                end;
            end

            entropcn = sum(entropc)/Li^2;
            entropsn = sum(entrops)/Li^2;

            entropiat = entropcn + entropsn;

            res = res + 1;

            Results(res).bits = bits;
            Results(res).epsilon = epsilon;
            Results(res).C = C;
            Results(res).entropia = entropiat;
            Results(res).Image = ImR;
            switch(variable)
                case 'SNR',
                    Results(res).SNR = SNRi;
                case 'MSE',
                    Results(res).MSE = mse;
                case 'SSIM',
                    Results(res).SSIM = esesim;
                case 'MPE lineal',
                    Results(res).MPE_lineal = mpelin;
                case 'MPE no lineal',
                    Results(res).MPE_no_lineal = mpenonlin;
                case 'MPE no lineal JMLR',
                    Results(res).MPE_no_lineal_JMLR = mpenonlinJMLR;
            end

        end
    end
end
