% ---------------------------------------------------------------------------------------
% SVR_dct calculates the rate-distortion curve for a picture (using
% constant tube until the cutoff frequency fc -en - cpd ).
%
% USO: [Results] = svr_dct(Im,bits,entropiad,epsilonv,kernel,perfil)
%
% Im              = image
% epsilon_inicial = epsilon starting values
% H               = SVR kernel
% perfil          = epsilon profile for each frequency
% sepsigno        = flag to pulling apart (or not) the signal sign in the entropy computation
%
%         Results(res).bits = bits;
%         Results(res).epsilon = epsilon;
%         Results(res).C = C;
%         Results(res).entropia = entropiat;
%         Results(res).Image = ImR;
%         Results(res).MSE = mse;
%         Results(res).SSIM = esesim;
%         Results(res).MPE_lin = mpelin;
%         Results(res).MPE_no_lin = mpenonlin;
%         Results(res).MPE_no_lin_JMLR = mpenonlinJMLR;
%
% To compute specific algorithms like RKi or SVR-cte, the epsilon profile
% has to be change inside the function.
% ---------------------------------------------------------------------------------------

function [Results] = svr_dct_epsilon_bits_entropy(Im,epsilon_inicial,H,perfil,sepsigno)

Lb=16;
Li=length(Im);

R=1:(Lb*Lb-1); R=R(:);
N=length(R);

C=40000;
D = ones(1,N); D = D(:);

res = 0;

Results=[];

ent_inicial = 0.05;

ent_final = 0.8;

ini_bits = 2;

fin_bits = 8;

for entropiad = ent_inicial:0.05:ent_final

    for bits = ini_bits:0.5:fin_bits

        ImR = rand(Li);

        entropiat = 10;
        umbral = 0.005;

        epsilon = 0;

        iter = 0;

        porcentaje = 1;

        sentido = 0;

        no_results = 0;

        l_min = 0;
        l_max = 100000;

        while (abs(entropiad - entropiat) > umbral) & (iter < 30)

            iter = iter + 1;

            signoscoefs=[];
            betasc=[];
            BtC=[];
            Coef11=[];
            SVs=[];
            lSVs=[];

            if(iter ~= 1 & iter ~= 2)

                [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon_entropy(entropiad, entropiat, epsilon, l_min, l_max, porcentaje, sentido);
            end

            fprintf('Training all the blocks of the local DCT');

            for i=1:Lb:Li

                fprintf('.');

                for j=1:Lb:Li

                    B=Im(i:(i+(Lb-1)),j:(j+(Lb-1)));
                    B=dct2(B);
                    Coef11=[Coef11;B(1,1)];
                    B=zigzag(B);
                    B=B(2:end);
                    BtC=[BtC;B'];

                    bitsig=getsign(B,Lb);
                    signoscoefs = [signoscoefs; bitsig];

                    [nsv,beta,bias] = irwls_pd_svr_nobias(R,abs(B),H,C,D,epsilon,perfil);

                    betasc=[betasc;beta'];

                end
            end

            fprintf('\n');

            MM=max(max(abs(betasc))); MM=MM/4;

            NumNiv = 2^bits;

            betascr = round( NumNiv * betasc/MM)*MM/NumNiv;

            signos = zeros(size(signoscoefs));
            signos(logical(betascr~=0))=signoscoefs(logical(betascr~=0));

            entropc = [];
            entrops = [];

            codigon = betascr.*(~signos) - betascr.*(signos);
            for i=1:Lb^2

                if sepsigno==0
                    codigo = betascr(i,:).*(~signos(i,:)) - betascr(i,:).*(signos(i,:));
                    [rlePesos,repCeros] = rle2(codigo);
                    entropc = [entropc length(rlePesos)*entropiv(rlePesos)+length(repCeros)*entropiv(repCeros)];
                    entrops = 0;
                else

                    [rlePesos,repCeros] = rle2(betascr(i,:));
                    entropc = [entropc length(rlePesos)*entropiv(rlePesos)+length(repCeros)*entropiv(repCeros)];

                    [rlePesoss,repCeross] = rle2(signos(i,:));
                    entrops = [entrops length(rlePesoss)*entropiv(rlePesoss)+length(repCeross)*entropiv(repCeross)];
                end;

            end

            entropcn = sum(entropc)/Li^2;
            entropsn = sum(entrops)/Li^2;

            entropiat = entropcn + entropsn;

            fprintf('   It. =

            if((iter == 1) & (entropiad - entropiat > 0) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits');
                break;

            elseif((iter == 1) & (abs(entropiad - entropiat) <= umbral) & (epsilon == 0))

                epsilon = epsilon_inicial;

                entropiat = 10;

            elseif(iter == 1)
                epsilon = epsilon_inicial;
            end
        end

        if(no_results == 0)

            z=1;

            for i=1:Lb:Li
                for j=1:Lb:Li

                    y=H*betascr(z,:)';

                    y=obtainsign(y, signos, Lb,z);

                    A=dezigzag([Coef11(z);y]);

                    A=idct2(A);

                    ImR(i:(i+(Lb-1)),j:(j+(Lb-1)))=A;

                    z=z+1;

                end
            end

            [mse,SNRi] = emse2(Im,ImR);

            esesim = ssim_index(Im,ImR);

            [MSE,d_l,dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(Im,ImR,2,[3 mean(mean(Im))],2,2);
            mpelin = d_l;

            [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],1,0.5,2,2);
            mpenonlinJMLR= d;

            [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],2,1,2,2);
            mpenonlin= d;

            res = res + 1;

            Results(res).bits = bits;
            Results(res).epsilon = epsilon;
            Results(res).C = C;
            Results(res).entropia = entropiat;
            Results(res).Image = ImR;
            Results(res).MSE = mse;
            Results(res).SSIM = esesim;
            Results(res).MPE_lin = mpelin;
            Results(res).MPE_no_lin = mpenonlin;
            Results(res).MPE_no_lin_JMLR = mpenonlinJMLR;

        end
    end
end
