% SVR_NL DCTs 'retransformadas' of a 2D signal subblock
% (Currently 256 * 256 resolution 4)
%
% Difference of QDCT63_2 in allowing the introduction of a mask
% (16 * 16 ) selecting certain coefficients so that ,
% increased sequentially, you can simulate the progressive decoding
% (sending bits... : see progressive_coding.m ) .
% This mask is introduced into the variable 'progres' .
% If progres = ones ( 16,16 ), the result is the same as using qdct63_2
%
% Difference of QDCT62 that allows you to not calculate the eigenvalues ​​of the original
% (if you're not interested, it saves time !)
%
% Represents QDCT62 signal in a domain with (approx.) statistically uncorrelated characteristics
% and uncorrelated perceptually (With perceptual metric identity).
% This is achieved through the divisive normalization: Teo & Heeger, Watson and Simoncelli.
% This transformation depends on its input signal ( state adaptation ) .
% It has been shown that this type of response obtains independent statistics ( Simoncelli99 ) and
% perceptually ( Malo99 ) coefficients, and therefore is a suitable domain for
% a scalar quantizer.
%
% Since the perceptual metric in this domain is the identity apply uniform quantizers
% and flat bit-allocation per coefficient.
%
% For each block the invertibility is checked [max( eig ( D_R * h )) ], and the inverse is computed
% by using the analytical expression
%
% It is normalized to local Michaelson contrast and response according NAKA8.m :
%
% Norm = [1 1] ................ Normalized by the average luminance
% of the subfield considered (local contrast) .
%
% Norm = [3 Lm ] ............... Normalized by the fixed value Lm (global contrast)
% the logical choice is to take Lm = mean ( mean (Im) );
% ( Mean value in digital levels )
%
%
% Range to distribute the quantization levels in the domain response is given
% by the maximum diagonal gradient ( acting on contrasts, belonging to
% [ -1.1 ] ) .
%
% USE: [Results] = svr_nl2(Im,bits,parametro,epsilon,K,exponente,sepsigno,variable)
%
% Im        -> Image to compress
% bits      -> Numerb of bits for the SVR weights quantization
% parametro -> Value parameter of distortion \ selected
% epsilon   -> Starting epsilon value
% K         -> SVR Kernel
% exponente -> Exponent for the nonlinearities
% Sepsigno  -> Flag to account for the entropy of signs ( '1 ') or
% 		no ( '0 ')
% Variable  -> Text string to select the distortion parameter.
%
% Results 	-> Vector of struct containing information about the results
% 		for the given parameter of distortion :
% 		Struct ( 'bits ' bits, 'epsilon' , epsilon , 'NumNiv', NumNiv,
%		'Image', ImR , 'Entropy', entropiat, 'Entropiasig', entropsn, 'entropiapes', entropcn)
% 		Results.SNR , Results.MSE , Results.SSIM , Results.MPE_lineal ,
% 		Results.MPE_no_lineal , Results.MPE_no_linea_JMLR

function Results = svr_nl_epsilon_bits_distortion(Im,param,epsilon_inicial,K,exponente,sepsigno,variable)

MM = 0.8;
normaliz = [3 mean(mean(Im))];
cero = 1;

bloques = [];
for i = 1:16
    for j = 1:16
        bloques = [bloques;i j];
    end
end

[h,alpha,beta,gamm] = constrains_resp(exponente,cero);

Ncuan = 4;
tam = size(Im);
tam = tam(1);
lados = [tam tam];
lcuan = tam/(2^Ncuan);

posai = [tam tam]/2-round(tam/2);
posbd = [tam tam]/2+round(tam/2);
coorcuanai = floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
coorcuanbd = floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
if coorcuanai(1) < 1
    coorcuanai(1) = 1;
end
if coorcuanai(2) < 1
    coorcuanai(2) = 1;
end
if coorcuanbd(1) > (2^Ncuan)
    coorcuanbd(1) = 2^Ncuan;
end
if coorcuanai(2) > (2^Ncuan)
    coorcuanbd(2) = 2^Ncuan;
end

dcti = dct2r(Im,Ncuan);
[dct_contr,Lm] = contras2(dcti,Ncuan,normaliz);
DC = Lm*16;

dctc1 = zeros(tam,tam);

codigo = [];
nceros = [];

R = (1:255);R=R(:);

C = 40000;
perfil_C = ones(255,1);

perfil_e = ones(255,1);

Lb = lcuan;
Li = tam;

Results = [];
ImR = rand(Li);

res = 0;

ini_bits = 2;

fin_bits = 8;

for parametro = param

    for bits = ini_bits:0.5:fin_bits

        switch(variable)
            case 'SNR', parametro_actual = 1; umbral = 0.05;
            case 'MSE', parametro_actual = 1; umbral = 0.5;
            case 'SSIM', parametro_actual = 0; umbral = 0.01;
            case 'MPE lineal', parametro_actual = 100; umbral = 0.01;
            case 'MPE no lineal', parametro_actual = 100; umbral = 0.01;
            case 'MPE no lineal JMLR', parametro_actual = 100; umbral = 0.1;
        end

        epsilon = 0;

        iter = 0;

        porcentaje = 1;

        if(isequal(variable,'SSIM'))
            sentido = 0;
        else
            sentido = 1;
        end

        no_results = 0;

        l_min = 0;
        l_max = 100000;

        disp(['Adjustment of epsilon to reach desired ' variable ':   ' num2str(parametro)]);

        while (abs(parametro - parametro_actual) > umbral) & (iter < 30)

            iter = iter + 1;
            dctc1 = zeros(tam,tam);
            dctsincuant = dctc1;
            signosr = [];
            betasc = [];
            betascr = [];
            dctc1i = [];

            if(iter ~= 1 & iter ~= 2)
                switch(variable)
                    case 'SNR',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon_SSIM(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MSE',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'SSIM',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon_SSIM(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MPE lineal',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MPE no lineal',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                    case 'MPE no lineal JMLR',
                        [epsilon, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro, parametro_actual, epsilon, l_min, l_max, porcentaje, sentido);
                end
            end

            fprintf('  Training all the blocks of the local non-linear domain');

            for i = 1:Lb:Li
                fprintf('.');
                for j = 1:Lb:Li

                    aa = dct_contr(i:(i+(Lb-1)),j:(j+(Lb-1)));

                    [h_a,r,GR,alfa2d,beta2d] = respue5(aa,alpha,beta,gamm,h,0);

                    r = zigzag(r);

                    [nsv,pesosSVM,bias] = irwls_pd_svr_nobias(R,r(2:256),K,C,perfil_C,epsilon,perfil_e);

                    azig = zigzag(aa);
                    azigsc = azig(2:end);

                    bitsig = getsign(azigsc,Lb);
                    signosr = [signosr; bitsig];
                    betasc = [betasc;pesosSVM'];
                end
            end
            fprintf('\n');

            NumNiv = 2^bits;

            betascr = round( NumNiv * betasc/MM)*MM/NumNiv;

            rc = [];
            for fila = 1:Li
                rc = [rc; (K*betascr(fila,:)')'];
            end

            signos = zeros(size(signosr));
            signos(logical(abs(rc)>0)) = signosr(logical(abs(rc)>0));

            fprintf('Recovering the image:\n');
            fprintf('  Inverting from r to dct\n');

            z = 1;
            DC = DC';
            DC = DC(:);

            Db = diag(beta2d);
            Identidad = eye(size(h));

            for i = 1:Lb:(Li-1)
                for j = 1:Lb:(Li-1)

                    r_recon = K*betascr(z,:)';

                    r_recon = [0; r_recon];

                    Dr = diag(r_recon);

                    ac = ((inv(Identidad-Dr*h)*Db*r_recon).^(1/gamm))./alfa2d;

                    ac1 = obtainsign(ac(2:end), signos, Lb,z);
                    ac1 = [0;ac1];

                    ac1 = dezigzag(ac1);

                    ac1(1,1) = DC(z);

                    dctc1(i:(i+(Lb-1)),j:(j+(Lb-1))) = ac1;

                    z = z+1;
                end
            end

            fprintf('  Inverting from dct to spatial domain\n');

            dctc1i = icontra2(dctc1,Lm,4,normaliz);
            ImR = real(idct2r(dctc1i,4));

            switch(variable)
                case 'SNR',
                    [mse,SNRi] = emse2(Im,ImR);
                    fprintf('   It. =
                    parametro_actual = SNRi;
                case 'MSE',
                    [mse,SNRi] = emse2(Im,ImR);
                    fprintf('   It. =
                    parametro_actual = mse;
                case 'SSIM',
                    esesim = ssim_index(Im,ImR);
                    fprintf('   It. =
                    parametro_actual = esesim;
                case 'MPE lineal',
                    [MSE,d_l,dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(Im,ImR,2,[3 mean(mean(Im))],2,2);
                    mpelin = d_l;
                    fprintf('   It. =
                    parametro_actual = mpelin;
                case 'MPE no lineal',
                    [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],2,1,2,2);
                    mpenonlin= d;
                    fprintf('   It. =
                    parametro_actual = mpenonlin;
                case 'MPE no lineal JMLR',
                    [MSE,d,dx,MPEx,MPEoo,MPEexpesp] = disdis7(Im,ImR,[3 mean(mean(Im))],1,0.5,2,2);
                    mpenonlinJMLR = d;
                    fprintf('   It. =
                    parametro_actual = mpenonlinJMLR;
            end

            if(~isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual < 0) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits');
                break;

            elseif(isequal(variable,'SSIM') & (iter == 1) & (parametro - parametro_actual > 0) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits');
                break;

            elseif((iter == 1) & (abs(parametro - parametro_actual) <= umbral) & (epsilon == 0))

                epsilon = epsilon_inicial;

                parametro_actual = 0;

            elseif(iter == 1)
                epsilon = epsilon_inicial;
            end

        end

        if(no_results == 0)

            entropc = [];
            entrops = [];

            codigon = betascr.*(~signos) - betascr.*(signos);
            for i = 1:Lb^2

                if sepsigno == 0
                    codigo = betascr(i,:).*(~signos(i,:)) - betascr(i,:).*(signos(i,:));
                    [rlePesos,repCeros] = rle2(codigo);
                    entropc = [entropc length(rlePesos)*entropiv(rlePesos)+length(repCeros)*entropiv(repCeros)];
                    entrops = 0;
                else

                    [rlePesos,repCeros] = rle2(betascr(i,:));
                    entropc = [entropc length(rlePesos)*entropiv(rlePesos)+length(repCeros)*entropiv(repCeros)];

                    [rlePesoss,repCeross] = rle2(signos(i,:));
                    entrops = [entrops length(rlePesoss)*entropiv(rlePesoss)+length(repCeross)*entropiv(repCeross)];
                end;

            end

            entropcn = sum(entropc)/Li^2;
            entropsn = sum(entrops)/Li^2;

            entropiat = entropcn + entropsn;

            res = res + 1;

            Results(res).bits = bits;
            Results(res).epsilon = epsilon;
            Results(res).C = C;
            Results(res).entropia = entropiat;
            Results(res).Image = ImR;

            switch(variable)
                case 'SNR',
                    Results(res).SNR = SNRi;
                case 'MSE',
                    Results(res).MSE = mse;
                case 'SSIM',
                    Results(res).SSIM = esesim;
                case 'MPE lineal',
                    Results(res).MPE_lineal = mpelin;
                case 'MPE no lineal',
                    Results(res).MPE_no_lineal = mpenonlin;
                case 'MPE no lineal JMLR',
                    Results(res).MPE_no_lineal_JMLR = mpenonlinJMLR;
            end
        end
    end
end
