
% DISDIS7 applies the perceptual distortion measure:
%
%    dx(rx,rx+Drx) = ( (abs(Drx').^(expfrec/2)) * (abs(Drx).^(expfrec/2)) )^(1/expfrec)
%
% to the responses, rx(Ax), of each subblock, Ax, of the given images, im, im+Dim.
% (16*16 blocks assuming fs=64 cpd)
%
% It employs bock-DCT as local frequential domain before the normalization
% No spatial interactions are employed between blocks...
%
% The spatial differences in each block, dx, are added spatially
% by using the norm expesp:
%
%                         /                \ ^ (1/expesp)
%       d(im,im+Dim) =   |  sum dx^(expesp) |
%                         \  x             /
%
%
% Moreover it gives a Maximum Perceptual Error map:
%
%    MPEx = max( abs(Drxf) )
%
%  and two average MPE:
%
%  - With infinity norm:
%
%     MPEoo  = max( MPEx )
%
%  - With expesp norm:
%
%                      /                  \ ^ (1/expesp)
%       MPEexpesp =   |  sum MPEx^(expesp) |
%                      \  x               /
%
% And also the MSE...
%
%
% USE: [MSE,d,dx,MPEx,MPEoo,MPEexpesp]=disdis7(im1,im2,[norm parametro],exponente_norm,fact_mask,expfrec,expesp);
%

function [MSE,d,dx,MPEx,MPEoo,MPEexpesp]=disdis7(im1,im2,norm,exponente,fact_mask,expfrec,expesp);

   tam=size(im1);
   tam=tam(1);

   lcuan=16;
   Ncuan=log(tam/lcuan)/log(2);
   lados=[tam tam];
   imq=zeros(tam,tam);
   dctq=zeros(tam,tam);
   codq=zeros(tam,tam);

   posai=[tam tam]/2-round(tam/2);
   posbd=[tam tam]/2+round(tam/2);
   coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
   coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
   if coorcuanai(1)<1
        coorcuanai(1)=1;
   end
   if coorcuanai(2)<1
        coorcuanai(2)=1;
   end
   if coorcuanbd(1)>(2^Ncuan)
        coorcuanbd(1)=2^Ncuan;
   end
   if coorcuanai(2)>(2^Ncuan)
        coorcuanbd(2)=2^Ncuan;

   end

   fs=64;
   cero=0;
   [H,kk1,kk2,gamm]=constrains_resp(exponente,cero);

   dcti1=dct2r(im1,Ncuan);
   [a1,Lm1]=contras2(dcti1,Ncuan,norm);
   DC1=Lm1*16;

   dcti2=dct2r(im2,Ncuan);
   [a2,Lm2]=contras2(dcti2,Ncuan,norm);
   DC2=Lm2*16;

dx=zeros(coorcuanbd(1),coorcuanbd(2));
MPEx=dx;

fprintf('Computing MPE distortion measure using a non-linear response model\n');

   for ii=coorcuanai(1):coorcuanbd(1)
       fprintf('.');
       for jj=coorcuanai(2):coorcuanbd(2)
           iii=(ii-1)*lcuan+ceil(lcuan/2);
           jjj=(jj-1)*lcuan+ceil(lcuan/2);
           blo1=sacasub(a1,[iii jjj],[lcuan lcuan],0);
           blo2=sacasub(a2,[iii jjj],[lcuan lcuan],0);

           [IN,R1,GR,alfa,beta]=respue5(blo1,kk1,fact_mask*kk2,gamm,H,0);
           [IN,R2,GR,alfa,beta]=respue5(blo2,kk1,fact_mask*kk2,gamm,H,0);

           Drf=abs(R1-R2);

           dx(ii,jj)= sum(sum((Drf.^(expfrec/2)).*(Drf.^(expfrec/2)))).^(1/expfrec);
           MPEx(ii,jj)=max(max(Drf));

       end
   end

fprintf('\n');

d=(sum(sum(dx.^(expesp))))^(1/expesp);

MPEexpesp=(sum(sum(MPEx.^(expesp))))^(1/expesp);
MPEoo=max(max(MPEx));

MSE = sum(sum( (im1-im2).^2 ))/(tam^2);
