%
% [MSE,d,dx,MPEx,MPEoo,MPEexpesp]=disdis5(im1,im2,naka_ngan,[norm parametro],expfrec,expesp);
%
% DISDIS_LINEAL applies the perceptual distortion measure:
%
%    dx(rx,rx+Drx) = ( (abs(Drx').^(expfrec/2)) * (abs(Drx).^(expfrec/2)) )^(1/expfrec)
%
% to the responses, rx(Ax), of each subblock, Ax, of the given images, im, im+Dim.
% (16*16 blocks assuming fs=64 cpd)
%
% It employs bock-DCT as local frequential domain before the normalization
% No spatial interactions are employed between blocks...
%
% This function applies the lineal model using a CSF ('naka100' or 'Ngan')
%
%     naka_ngan = 1 ..... naka
%     naka_ngan = 2 ..... ngan
%
% The spatial differences in each block, dx, are added spatially
% by using the norm expesp:
%
%                         /                \ ^ (1/expesp)
%       d(im,im+Dim) =   |  sum dx^(expesp) |
%                         \  x             /
%
%
% Moreover it gives a Maximum Perceptual Error map:
%
%    MPEx = max( abs(Drxf) )
%
%  and two average MPE:
%
%  - With infinity norm:
%
%     MPEoo  = max( MPEx )
%
%  - With expesp norm:
%
%                      /                  \ ^ (1/expesp)
%       MPEexpesp =   |  sum MPEx^(expesp) |
%                      \  x               /
%
% And also the MSE...
%
%

function [MSE,d,dx,MPEx,MPEoo,MPEexpesp]=disdis_lineal(im1,im2,naka_ngan,norm,expfrec,expesp);

tam=size(im1);
tam=tam(1);

lcuan=16;
Ncuan=log(tam/lcuan)/log(2);
lados=[tam tam];
imq=zeros(tam,tam);
dctq=zeros(tam,tam);
codq=zeros(tam,tam);

posai=[tam tam]/2-round(tam/2);
posbd=[tam tam]/2+round(tam/2);
coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
if coorcuanai(1)<1
coorcuanai(1)=1;
end
if coorcuanai(2)<1
coorcuanai(2)=1;
end
if coorcuanbd(1)>(2^Ncuan)
coorcuanbd(1)=2^Ncuan;
end
if coorcuanai(2)>(2^Ncuan)
coorcuanbd(2)=2^Ncuan;

end

fs=64;

if naka_ngan==1
kk1 = [0.0034    0.0194    0.0351    0.0316    0.0197    0.0151    0.0138    0.0131    0.0116    0.0092  0.0069    0.0050    0.0035    0.0032    0.0032    0.0032];
else
[kk1,lala]=ngan(32,16);
end

fact=0.5;

fx=linspace(0,fs/2-fs/(2*lcuan),lcuan);

Fx=ones(lcuan,1)*fx;
Fy=Fx';
F=sqrt(Fx.^2+Fy.^2);

fx=zigzag(Fx);
fy=zigzag(Fy);
f=zigzag(F);

dcti1=dct2r(im1,Ncuan);
[a1,Lm1]=contrast_2(dcti1,Ncuan,norm);
DC1=Lm1*16;

dcti2=dct2r(im2,Ncuan);
[a2,Lm2]=contrast_2(dcti2,Ncuan,norm);
DC2=Lm2*16;

dx=zeros(coorcuanbd(1),coorcuanbd(2));
MPEx=dx;

pes=ones(16,16);
pes(1,1)=0;

fprintf('Computing MPE distortion measure using a linear response model\n');

for ii=coorcuanai(1):coorcuanbd(1)
fprintf('.');
for jj=coorcuanai(2):coorcuanbd(2)

   iii=(ii-1)*lcuan+ceil(lcuan/2);
   jjj=(jj-1)*lcuan+ceil(lcuan/2);
   blo1=sacasub(a1,[iii jjj],[lcuan lcuan],0);
   blo2=sacasub(a2,[iii jjj],[lcuan lcuan],0);

   [R1,CSF]=respue_lineal(blo1,kk1/max(kk1));
   [R2,CSF]=respue_lineal(blo2,kk1/max(kk1));

   R1=pes.*R1;
   R2=pes.*R2;

   Drf=abs(R1-R2);

   dx(ii,jj)= sum(sum((Drf.^(expfrec/2)).*(Drf.^(expfrec/2)))).^(1/expfrec);
   MPEx(ii,jj)=max(max(Drf));

end
end

fprintf('\n');

d=(sum(sum(dx.^(expesp))))^(1/expesp);

MPEexpesp=(sum(sum(MPEx.^(expesp))))^(1/expesp);
MPEoo=max(max(MPEx));

MSE = sum(sum( (im1-im2).^2 ))/(tam^2);
