%
% RESPUE5 compute the response ( and gradient ) to a local DCT2D
% (expressed in contrast) considering interactions between the coefficients,
% simulating the HVS frequency detectors.
%
% DCTN (f) = ( k1 (f) DCT (f) ) ^ g / (k2 (f) + [ H * ( k1 (f) DCT (f) ) ^ g ] (f) )
%
% Inputs:
% H, k1, k2 (only frequency dependence) and g,
%
% Returns :
%
% interac = interaction = H * abs ( a) ^ g
% Resp = response,
% Grad = The Jacobian of the response
% If you decide not to calculate the Jacobian , the program sets GR = 0
% The parameters k1 and k2 in a matrix form , KK1 , KK2
%
% NOTE : It is assumed that the block gets into the function is already expressed in
% contrast ( see CONTRAST)
%
% USE : [interac,Resp,Grad,kk1,kk2]=respue5(dctinic,k1,k2,gamm,H,calc_grad)
%

function [IN,R,GR,alfa,beta]=respue5(dctinic,k1,k2,gamm,H,calc_grad)

   tam=size(dctinic);
   tam=tam(1);
   lados=[tam tam];

   lcuan=lados(1);

   k1=simrev(k1);
   k1=zigzag(k1);
   k2=simrev(k2);
   k2=zigzag(k2);

   alfa=k1;
   beta=k2;

cub=dctinic;

cub=zigzag(cub);

cub=abs(cub);

cucu=(k1.*cub).^gamm;

interacc=H*[0;cucu(2:length(cucu))];

R=cucu./(k2+interacc);
R(1)=sqrt(cub(1));

if calc_grad==1

	I=eye(lcuan^2);
	GR=zeros(lcuan^2,lcuan^2);
	GR(1,1)=1;

	for i=2:lcuan^2,
	  for j=2:lcuan^2,

	      GR(i,j)= gamm*(abs(  (cucu(i).^(1/gamm)).^(gamm-1)  )/(k2(i)+interacc(i))*I(i,j)-(cucu(i)*abs( ((cucu(j)).^(1/gamm)).^(gamm-1) )/((k2(i)+interacc(i))^2))*H(i,j));
	  end
	end

else
	GR=0;
end

R=dezigzag(R);
IN=dezigzag(interacc);
