% CONTRAST normalized amplitudes of local DCT resolution N
% We can choose different normalizations :
%
% Norm = 1 ......... Normalized by the average luminance of the subblock considered.
%
% Norm = 2 ......... Normalized by the average luminance of the subblock considered and
% of the subblock of neighbourhood. The number of neighbourhood sub-blocks must be
% entered as a parameter .
%
% Norm = 3 ......... Normalized by the average luminance of the entire image.
%
% Norm = 4 ......... Normalized by the value ( luminance ) introduced as a parameter
% ( Value * L ^ 2 ) , where l is the side of subblock local DCTs .
%
%
% NOTE : Speaking of luminance we mean the extent to which the expressed input image.
% Except calibration, the input image is usually expressed in 'gray levels '
% not strictly in luminance .
%
% Normalizations 1 and 3
%
% USE:
% [DCTF,Lm] = contrast_2(DSTI,Ncuan,[Norm parameter]);
%

function [dctf,Lm]=contrast_2(s,Ncuan,norm)

   par=norm(2);
   norm=norm(1);
   tam=size(s);
   tam=tam(1);
   lados=[tam tam];

   dctf=zeros(tam,tam);

   lcuan=tam/(2^Ncuan);
   posai=[tam tam]/2-round(tam/2);
   posbd=[tam tam]/2+round(tam/2);
   coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
   coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
   if coorcuanai(1)<1
        coorcuanai(1)=1;
   end
   if coorcuanai(2)<1
        coorcuanai(2)=1;
   end
   if coorcuanbd(1)>(2^Ncuan)
        coorcuanbd(1)=2^Ncuan;
   end
   if coorcuanai(2)>(2^Ncuan)
        coorcuanbd(2)=2^Ncuan;
   end

   for ii=coorcuanai(1):coorcuanbd(1)
       for jj=coorcuanai(2):coorcuanbd(2)

           iii=(ii-1)*lcuan+ceil(lcuan/2);
           jjj=(jj-1)*lcuan+ceil(lcuan/2);
           blo=sacasub(s,[iii jjj],[lcuan lcuan],0);

           cub=blo;

           fla=0;
           if cub(1,1)==0
              fla=1;
              cub(1,1)=10;
           end
              if norm==1
                 cub(2:lcuan,1)=2*cub(2:lcuan,1)/cub(1,1);
                 cub(1,2:lcuan)=2*cub(1,2:lcuan)/cub(1,1);
                 cub(2:lcuan,2:lcuan)=4*cub(2:lcuan,2:lcuan)/cub(1,1);
                 Lm(ii,jj)=cub(1,1)/lcuan;
              elseif norm==2
                 Lmed=[];
                 for k=-par:par
                     for l=-par:par
                         cubil=sacasub(s,[iii+k jjj+l],[lcuan lcuan],0);
                         med=mean(mean(cubil));
                         if med>0
                            Lmed=[Lmed med];
                         end
                     end
                 end
                 Lmedia=4*mean(Lmed)*lcuan^2;
                 cub(2:lcuan,1)=2*cub(2:lcuan,1)/Lmedia;
                 cub(1,2:lcuan)=2*cub(1,2:lcuan)/Lmedia;
                 cub(2:lcuan,2:lcuan)=4*cub(2:lcuan,2:lcuan)/Lmedia;
              elseif norm==3
                 Lmedia=par;
                 cub11=Lmedia*lcuan;
                 cub(2:lcuan,1)=2*cub(2:lcuan,1)/cub11;
                 cub(1,2:lcuan)=2*cub(1,2:lcuan)/cub11;
                 cub(2:lcuan,2:lcuan)=4*cub(2:lcuan,2:lcuan)/cub11;
                 Lm(ii,jj)=cub(1,1)/lcuan;
              elseif norm==4
                 cub(2:lcuan,1)=2*cub(2:lcuan,1)/(4*par*lcuan^2);
                 cub(1,2:lcuan)=2*cub(1,2:lcuan)/(4*par*lcuan^2);
                 cub(2:lcuan,2:lcuan)=4*cub(2:lcuan,2:lcuan)/(4*par*lcuan^2);
              else
                 cub(2:lcuan,1)=cub(2:lcuan,1)/2;
                 cub(1,2:lcuan)=cub(1,2:lcuan)/2;
              end

           dctf=metesub(dctf,[iii jjj],cub);

       end
   end
