
%
%	ADAPTIVE SVR USING IRWLS WITH NO BIAS.
%	VERSION 2: SOLVE ONLY THE MINIMIZATION WRT W
%
%  [nsv,al3,bi]=irwls(x,y,ker,C,tol,epsi);
%
%  Input parameters:
%  	 x: input vectors.
%  	 y: output samples for x.
%  	 H: kernel matrix.
%  	 C: Penalization constraint.
%        D: C profile
%  	 epsi: insensitivity zone.
%        F: epsilon profile
%
%  Output parameters:
%   	 nsv: number of support vectors
%   	 al3: dual weights
%   	 bi: bias term

function [nsv,al3,bi]=irwls_pd_svr_nobias(x,y,H,C,D,epsi,F);

T=clock;
if (nargin<7 | nargin>7)
    help irwls_adapta_nobias2
else

    tol=10^-4;
    K=10^8;
    val_a=100;
    N=length(x);
    ns=-1;
    k=1;
    hacer=1;
    bi=0;

    i1p=1:2:N;
    i1n=2:2:N;
    i1=[i1p i1n];

    epsi = epsi./F;

    a(i1)=C*D(i1);
    al3=zeros(N,1);
    bi=0;
    i2p=[];
    i2n=[];
    e=y;

    while(hacer)
        al3_a=al3;
        bi_a=bi;
        al3=zeros(N,1);
        al3(i2p)=C*D(i2p);
        al3(i2n)=-C*D(i2n);

        if(length(i1))

            Xi=inv(H(i1,i1)+diag(1./(a(i1))));
            E=ones(N,1);
            E(i1n)=-1;

            if (length(i2n)==0) & (length(i2p)==0)
                aux=Xi*(y(i1)-E(i1).*epsi(i1));
            elseif (length(i2n)==0) & (length(i2p)~=0)
                aux=Xi*(y(i1)-E(i1).*epsi(i1)-C*H(i1,i2p)*D(i2p));
            elseif (length(i2n)~=0) & (length(i2p)==0)
                aux=Xi*(y(i1)-E(i1).*epsi(i1)+C*H(i1,i2n)*D(i2n));
            else
                aux=Xi*(y(i1)-E(i1).*epsi(i1)+C*H(i1,i2n)*D(i2n)-C*H(i1,i2p)*D(i2p));
            end;

            al3(i1)=aux(1:length(i1));

            bi=0;
        end

        e_a=e;

        I=find(al3-al3_a);
        if(length(I))
            e=e_a-H(:,I)*(al3(I)-al3_a(I));
        end

        if((max(abs(e-e_a))<tol & k>(ns+1)) | k>=2000)
            nsv=length(i1)+length(i2n)+length(i2p);
            hacer=0;
        else

            i1p=find(e>epsi);
            i1n=find(e<-epsi);
            i2p=find(e>epsi & al3==C*D);
            i2n=find(e<-epsi & al3==-C*D);

            a=zeros(N,1);
            a(i1p)=C*D(i1p)./(e(i1p)-epsi(i1p));
            a(i1n)=-C*D(i1n)./(e(i1n)+epsi(i1n));
            a(find(a>K))=K;

            i2p_c=find(al3>=(C*D-C*D/100) & al3<=(C*D+C*D/100) & e>epsi & a<val_a);
            i2p_c=setdiff(i2p_c',i2p)';
            i2p=sort([i2p' i2p_c'])';
            i1p=setdiff(i1p',i2p)';

            i2n_c=find(al3>=(-C*D-C*D/100) & al3<=(-C*D+C*D/100) & e<-epsi & a<val_a);
            i2n_c=setdiff(i2n_c',i2n)';
            i2n=sort([i2n' i2n_c'])';
            i1n=setdiff(i1n',i2n)';

            i1=[i1p;i1n];

            if(length(i1)==0)
                [val,pos]=min(e(i2p));
                i1p=i2p(pos);
                i2p=setdiff(i2p',i1p)';
                a(i1p)=C*D(i1p)./(e(i1p)-epsi(i1p));

                [val,pos]=max(e(i2n));
                i1n=i2n(pos);
                i2n=setdiff(i2n',i1n)';
                a(i1n)=C*D(i1n)./(e(i1n)-epsi(i1n));
            end

            k=k+1;
            if(rem(k,20)==0 & max(abs(e-e_a))<(tol*10))
                val_a=min(val_a*2,1000);
            end
        end
    end
end
