
% It computes the RMSE, SSIM, lineal MPE, non-lineal MPE (param. JMLR)
% of a distorted image.
%
% Im    -> Original image
% ImR   -> Distorted image
% pesos -> Used in RGB images. Is a row vector with the weights for each YUV layer
%
% In chromatic images, MPE's does not need a previous YUV transform
%
% USE: [RMSE,SSIM,MPEl,MPEnlJMLR] = computing_distortions(Im,ImR,pesos)

function [RMSE,SSIM,MPEl,MPEnlJMLR,MPEnlJMLRn] = computing_distortions(Im,ImR,pesos)
if(nargin < 3)
    pesos = 1;
end
N = size(Im,3);
SSIMn = zeros(N,1);
MPEln = zeros(N,1);
MPEnlJMLRn = zeros(N,1);
SSIM = 0;
MPEl = 0;
MPEnlJMLR = 0;
if(N == 3)

    ImYUV = my_rgb2yuv(double(Im));
    ImRYUV = my_rgb2yuv(double(ImR));
end
for capa = 1:N

    SSIMn(capa) = ssim_index(double(Im(:,:,capa)),ImR(:,:,capa));
    if(N == 3)

        [MSE,MPEln(capa),dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(double(ImYUV(:,:,capa)),ImRYUV(:,:,capa),2,[3 mean(mean(ImYUV(:,:,capa)))],2,2);

        %[MSE,MPEnlJMLRn(capa),dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(ImYUV(:,:,capa)),ImRYUV(:,:,capa),[3 mean(mean(ImYUV(:,:,capa)))],1,0.5,2,2);
        [MSE,MPEnlJMLRn(capa),dx,MPEx,MPEoo,MPEexpesp]=disdis7c(double(ImYUV(:,:,capa)),ImRYUV(:,:,capa),capa,[3 mean(mean(ImYUV(:,:,capa)))],1,0.5,2,2);


        
    else

        [MSE,MPEln(capa),dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(double(Im),ImR,2,[3 mean(mean(Im))],2,2);

        [MSE,MPEnlJMLRn(capa),dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im),ImR,[3 mean(mean(Im))],1,0.5,2,2);
    end
end
for i = 1:N

    SSIM = SSIM + pesos(i) * SSIMn(i);

    MPEl = MPEl + pesos(i) * MPEln(i);

    MPEnlJMLR = MPEnlJMLR + pesos(i) * MPEnlJMLRn(i);
end
RMSE = sqrt( sum((Im(:) - ImR(:)) .^ 2) / numel(Im) );
