% Obtains the parameters for the compression algorithm and the values for the
% distortion algorithm.
%
% perfil    -> Epsilon profile
% H         -> SVR Kernel (SVR-CTE, SVR-RKi, SVR-CSF,
%              SVR-NL exp ~= 1, SVR-NL exp = 2).
% exponente -> Exponent for the nonlinearity (SVR-NL exp ~= 1, SVR-NL exp = 2).
%
%
% Algorithms:
%
% 1  -> CTE + SVR
% 2  -> RKi + SVR
% 3  -> CSF + SVR
% 4  -> NL1 + SVR
% 5  -> NL2 + SVR
% 6  -> JPEG91
% 7  -> Malo99
% 8  -> Epifanio03
% 9  -> Malo06_1
% 10 -> Malo06_2
%
% USE: [perfil,H,exponente] = computing_parameters_entropy(algoritmo)

function [perfil,H,exponente] = computing_parameters_entropy(algoritmo)

N = 16;

switch(algoritmo)
    case 1,
        perfil = ones(1,N^2 - 1)';
        H = kernel2d_variance(N,1);
        exponente = [];
    case 2,

        fc = 18;
        perfil = low_pass(N,fc);
        perfil = perfil(2:end);
        H = kernel2d_variance(N,1);
        exponente = [];
    case 3,
        perfil = CSF(N);
        perfil = perfil(2:end);
        H = kernel2d_variance(16,1);
        exponente = [];
    case 4,
        perfil = [];
        H = kernel2d_variance(16,1);
        exponente = 1;
    case 5,
        perfil = [];
        H = kernel2d_variance(16,1);
        exponente = 2;
    case 6,
        exponente = [];
        perfil = [];
        H = [];
    case 7,
        exponente = [];
        perfil = [];
        H = [];
    case 8,
        perfil = [];
        H = [];
        exponente = [];
    case 9,
        perfil = [];
        H = [];
        exponente = 1;
    case 10,
        perfil = [];
        H = [];
        exponente = 2;
end

