
% Given a value for a distortion (RMSE o SSIM) it iterates to reach
% an image with this distortion.
% It encodes the image, gets a  "ristra" and decodes the image using
% the nl+svr algorithm.
% It computes the distortion and reset the epsilon parameter if necessary,
% and starts again.
% It gives back an struct with the encoder results and the decoded image.
%
% NOTE THAT: The distortion values vector has to be sorted descent for all the
% distortions but for the SSIM, which will sort in ascent.

function [Results] = distortion_svr(algoritmo,variable,distorsiones,Im,epsilon_ini,K,perfil,exponente,directorio)
N = length(Im);

ini_bits = 2.5;
fin_bits = 15;
for i = 1:length(distorsiones)

    ImR = [];
    disp(['Ajustando el epsilon para hallar el ' variable ' de:   ' num2str(distorsiones(i))]);
    for b = ini_bits:0.5:fin_bits
        switch(variable)
            case 'RMSE', distorsion_actual = 0; umbral = 0.01;
            case 'SSIM', distorsion_actual = 0; umbral = 0.001;
            case 'MPE lineal', distorsion_actual = 100; umbral = 0.01;
            case 'MPE no lineal', distorsion_actual = 100; umbral = 0.01;
            case 'MPE no lineal JMLR', distorsion_actual = 100; umbral = 0.1;
        end

        epsilon = 0;

        bits = b;

        iter = 0;

        porcentaje = 1;

        if(isequal(variable,'SSIM'))
            sentido = 0;
        else
            sentido = 1;
        end

        l_min = 0;
        l_max = 100000;

        no_results = 0;
        while((abs(distorsiones(i) - distorsion_actual) > umbral) & (iter < 30))
            iter = iter + 1;

            if(iter ~= 1 & iter ~= 2)
                if(isequal(variable,'SSIM'))
                    [epsilon,l_min,l_max,porcentaje,sentido] = fitting_epsilon_SSIM(distorsiones(i),distorsion_actual,epsilon,l_min,l_max,porcentaje,sentido);
                else
                    [epsilon,l_min,l_max,porcentaje,sentido] = fitting_epsilon(distorsiones(i),distorsion_actual,epsilon,l_min,l_max,porcentaje,sentido);
                end
            end

            switch(algoritmo)
                case 1,
                    fichero = 'cte_svr_distorsion';
                    svr_dct_encoder(Im,bits,epsilon,K,perfil,directorio,[fichero '_' num2str(i)]);
                case 2,
                    fichero = 'rki_svr_distorsion';
                    svr_dct_encoder(Im,bits,epsilon,K,perfil,directorio,[fichero '_' num2str(i)]);
                case 3,
                    fichero = 'csf_svr_distorsion';
                    svr_dct_encoder(Im,bits,epsilon,K,perfil,directorio,[fichero '_' num2str(i)]);
                case 4,
                    fichero = 'nl1_svr_distorsion';
                    svr_nl_encoder(Im,bits,epsilon,K,exponente,directorio,[fichero '_' num2str(i)]);
                case 5,
                    fichero = 'nl2_svr_distorsion';
                    svr_nl_encoder(Im,bits,epsilon,K,exponente,directorio,[fichero '_' num2str(i)]);
            end

            switch(algoritmo)
                case 1,
                    [ImR] = svr_dct_decoder(K,directorio,[fichero '_' num2str(i)]);
                case 2,
                    [ImR] = svr_dct_decoder(K,directorio,[fichero '_' num2str(i)]);
                case 3,
                    [ImR] = svr_dct_decoder(K,directorio,[fichero '_' num2str(i)]);
                case 4,
                    [ImR] = svr_nl_decoder(K,exponente,directorio,[fichero '_' num2str(i)]);
                case 5,
                    [ImR] = svr_nl_decoder(K,exponente,directorio,[fichero '_' num2str(i)]);
            end

            switch(variable)
                case 'RMSE',
                    [mse,SNRi] = emse2(double(Im),ImR);
                    distorsion_actual = sqrt(mse);
                case 'SSIM',
                    esesim = ssim_index(double(Im),ImR);
                    distorsion_actual = esesim;
                case 'MPE lineal',
                    [MSE,MPEl,dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(double(Im),ImR,2,[3 mean(mean(Im))],2,2);
                    distorsion_actual = MPEl;
                case 'MPE no lineal',
                    [MSE,MPEnl,dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im),ImR,[3 mean(mean(Im))],2,1,2,2);
                    distorsion_actual = MPEnl;
                case 'MPE no lineal JMLR',
                    [MSE,MPEnlJMLR,dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im),ImR,[3 mean(mean(Im))],1,0.5,2,2);
                    distorsion_actual = MPEnlJMLR;
            end
            disp(['Iteracion:  ' num2str(iter) '   ' variable ':  ' num2str(distorsion_actual) '   bits:  ' num2str(bits) '   epsilon:  ' num2str(epsilon)]);

            if(~isequal(variable,'SSIM') & (iter == 1) & (distorsiones(i) - distorsion_actual < 0) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits.');
                break;

            elseif(isequal(variable,'SSIM') & (iter == 1) & (distorsiones(i) - distorsion_actual > 0) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits.');
                break;

            elseif((iter == 1) & (abs(distorsiones(i) - distorsion_actual) <= umbral) & (epsilon == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits.');
                break;

            elseif(iter == 1)
                epsilon = epsilon_ini;
            end
        end
        if(no_results == 0)

            cd(directorio)
            unzip([fichero '_' num2str(i)]);
            if(algoritmo >= 4)
                ristra = read_code_svr_nl([fichero '_' num2str(i) '.bin']);
                Results(i).Exponente = exponente;
            else
                ristra = read_code_svr_dct([fichero '_' num2str(i) '.bin']);
            end

            delete([fichero '_' num2str(i) '.bin']);

            D = dir([directorio fichero '_' num2str(i) '.zip']);
            tamanyo = 8 * D.bytes;

            entropia = tamanyo / N^2;

            Results(i).Bits = bits;
            Results(i).Epsilon = epsilon;
            Results(i).Kernel = K;
            Results(i).Entropy = entropia;
            Results(i).Image = ImR;
            Results(i).ristra = ristra;
            switch(variable)
                case 'RMSE',
                    Results.RMSE = distorsion_actual;
                case 'SSIM',
                    Results.SSIM = distorsion_actual;
                case 'MPE lineal',
                    Results.MPE_lineal = distorsion_actual;
                case 'MPE no lineal',
                    Results.MPE_no_lineal = distorsion_actual;
                case 'MPE no lineal JMLR',
                    Results.MPE_no_lineal_JMLR = distorsion_actual;
            end

            ini_bits = b;

            break;
        end
    end
end
