
% Given a value for a distortion (RMSE o SSIM) it iterates to reach
% an image with this distortion.
% It encodes the image, gets a  "sequence" and decodes the image using
% the JPEG91, Malo99, Epifanio03, Malo06 algorithm.
% It computes the distortion and reset the uCP parameter if necessary,
% and starts again.
% It gives back an struct with the encoder results and the decoded image.
%
% NOTE THAT: The distortion values vector has to be sorted descent for all the
% distortions but for the SSIM, which will sort in ascent.

function [Results] = distortion_ucp(algoritmo,variable,distorsiones,Im,uCP_ini,exponente,directorio)
N = length(Im);
for i = 1:length(distorsiones)

    ImR = [];
    disp(['Ajustando el epsilon para hallar el ' variable ' de:   ' num2str(distorsiones(i))]);
    switch(variable)
        case 'RMSE', distorsion_actual = 0; umbral = 0.01;
        case 'SSIM', distorsion_actual = 0; umbral = 0.001;
        case 'MPE lineal', distorsion_actual = 100; umbral = 0.01;
        case 'MPE no lineal', distorsion_actual = 100; umbral = 0.01;
        case 'MPE no lineal JMLR', distorsion_actual = 100; umbral = 0.1;
    end

    uCP = 0;

    iter = 0;

    porcentaje = 1;

    if(isequal(variable,'SSIM'))
        sentido = 0;
    else
        sentido = 1;
    end

    l_min = 0;
    l_max = 100000;

    no_results = 0;
    while((abs(distorsiones(i) - distorsion_actual) > umbral) & (iter < 30))
        iter = iter + 1;

        if(iter ~= 1 & iter ~= 2)
            if(isequal(variable,'SSIM'))
                [uCP,l_min,l_max,porcentaje,sentido] = fitting_uCP_SSIM(distorsiones(i),distorsion_actual,uCP,l_min,l_max,porcentaje,sentido);
            else
                [uCP,l_min,l_max,porcentaje,sentido] = fitting_uCP(distorsiones(i),distorsion_actual,uCP,l_min,l_max,porcentaje,sentido);
            end
        end

        switch(algoritmo)
            case 6,
                fichero = 'jpeg91_distorsion';
                jpeg91_encoder(Im,uCP,directorio,[fichero '_' num2str(i)]);
            case 7,
                fichero = 'malo99_entropia';
                malo99_encoder(Im,uCP,directorio,[fichero '_' num2str(i)]);
            case 8,
            case 9,
                fichero = 'malo06_1_distorsion';
                malo06_encoder(Im,uCP,exponente,directorio,[fichero '_' num2str(i)]);
            case 10,
                fichero = 'malo06_2_distorsion';
                malo06_encoder(Im,uCP,exponente,directorio,[fichero '_' num2str(i)]);
        end

        switch(algoritmo)
            case 6,
                [ImR] = jpeg91_decoder(directorio,[fichero '_' num2str(i)]);
            case 7,
                [ImR] = malo99_decoder(directorio,[fichero '_' num2str(i)]);
            case 8,
            case 9,
                [ImR] = malo06_decoder(exponente,directorio,[fichero '_' num2str(i)]);
            case 10,
                [ImR] = malo06_decoder(exponente,directorio,[fichero '_' num2str(i)]);
        end

        switch(variable)
            case 'RMSE',
                [mse,SNRi] = emse2(double(Im),ImR);
                distorsion_actual = sqrt(mse);
            case 'SSIM',
                esesim = ssim_index(double(Im),ImR);
                distorsion_actual = esesim;
            case 'MPE lineal',
                [MSE,MPEl,dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(double(Im),ImR,2,[3 mean(mean(Im))],2,2);
                distorsion_actual = MPEl;
            case 'MPE no lineal',
                [MSE,MPEnl,dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im),ImR,[3 mean(mean(Im))],2,1,2,2);
                distorsion_actual = MPEnl;
            case 'MPE no lineal JMLR',
                [MSE,MPEnlJMLR,dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im),ImR,[3 mean(mean(Im))],1,0.5,2,2);
                distorsion_actual = MPEnlJMLR;
        end
        disp(['Iteracion:  ' num2str(iter) '   ' variable ':  ' num2str(distorsion_actual) '   bits:  ' num2str(bits) '   epsilon:  ' num2str(epsilon)]);

        if(~isequal(variable,'SSIM') & (iter == 1) & (distorsiones(i) - distorsion_actual > 0) & (uCP == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP.');
            break;

        elseif(isequal(variable,'SSIM') & (iter == 1) & (distorsiones(i) - distorsion_actual < 0) & (uCP == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP.');
            break;

        elseif((iter == 1) & (abs(distorsiones(i) - distorsion_actual) <= umbral) & (uCP == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP.');
            break;

        elseif(iter == 1)
            uCP = uCP_ini;
        end
    end
    if(no_results == 0)

        cd(directorio)
        unzip([fichero '_' num2str(i)]);
        if(algoritmo >= 9)
            ristra = read_code_malo06([fichero '_' num2str(i) '.bin']);
            Results(i).Exponente = exponente;
        elseif(algoritmo == 6)
            ristra = read_code_jpeg91([fichero '_' num2str(i) '.bin']);
        elseif(algoritmo == 7)
            ristra = read_code_malo99([fichero '_' num2str(i) '.bin']);

        end

        delete([fichero '_' num2str(i) '.bin']);

        D = dir([directorio fichero '_' num2str(i) '.zip']);
        tamanyo = 8 * D.bytes;

        entropia = tamanyo / N^2;

        Results(i).uCP = uCP;
        Results(i).Entropy = entropia;
        Results(i).Image = ImR;
        Results(i).ristra = ristra;
        switch(variable)
            case 'RMSE',
                Results(i).RMSE = distorsion_actual;
            case 'SSIM',
                Results(i).SSIM = distorsion_actual;
            case 'MPE lineal',
                Results(i).MPE_lineal = distorsion_actual;
            case 'MPE no lineal',
                Results(i).MPE_no_lineal = distorsion_actual;
            case 'MPE no lineal JMLR',
                Results(i).MPE_no_lineal_JMLR = distorsion_actual;
        end
    end
end
