% SVR_NL_C_ENCODER encoding images by quantization of the SVM weights trained in the DCT domain
% of the chromatic YUV channels transformed by divisive normalization (currently 256 * 256 images)
%
% The DC components are coded by DPCM in YUV.
%
% Before applying non - linear responses DCTs must be expressed in
% contrasts (chromatic and achromatic) by using contras3.m
% This function allows three modes of normalization:
%
% Norm = 1 ................ Calculates contrast in each block by reference to the average color of
% the whole picture.
%
% Norm = 2 ................ Calculates contrast in each block by reference to the average color of
% that block.
%
% Norm = 3 ................ Calculates contrast in each block with reference to color ( RGB NTSC)
% that is passed. ( in the other cases put To = 0)
%
% To calculate the chromatic contrasts we have to define what we
% call Contrast unit (set the maximum variation of the U -channel and
% V , and Dumax dVmax). It is reasonable to set these parameters around 50.
%
% Uniform quantization SVMs weights of each block of each channel is determined
% by a number of bits for each channel. The number of support vectors
% depends on the epsilon-insensitivity that is set for the Machine.
%
% K_svm is the kernel used in the machines. This is a rbf kernel
% K = kernel2d_variance(16,1);
%
% The parameter 'exponent' contains the exponents of excitement and
% inhibition for each channel. There are two options:
%
% Exp = 2 ... better results but more unstable inverse.
% Exp = 1 ... worse but more robust inverse.
%
% You can choose different CSFs for the achromatic channel:
% Tipo_alfa = 1 Selects adjusted CSF
% Tipo_alfa = 2 Selects Ngan CSF
% Tipo_alfa = 3 Selects Kelly CSF (as earlier versions of JPEG)
%
% USE: [Results]=svr_nl_c_encoder(Im,norm,To,dUmax,dVmax,bits,epsilonv,K_svm,exponente,tipo_alfa,directorio,fichero,RESP))
%

function RESP=svr_nl_c_encoder_cs(Im,normaliz,To,dUmax,dVmax,bits,epsilonv,K,exponente,tipo_alfa,directorio,fichero,RESP)

you_are_here=pwd;

Im=double(Im);

if nargin<13
   RESP=[];
end

MM = 0.8;

cero = 1;

Ncuan=4;
tam=size(Im);
if length(tam)==3
    capas=tam(3);
else
    capas=1;
end

tam=tam(1);
lados=[tam tam];
lcuan=tam/(2^Ncuan);

YUV=my_rgb2yuv(double(Im));

for i=1:capas
    dcti(:,:,i)=dct2r(YUV(:,:,i),Ncuan);
end

[dct_contr,Lm]=contras3(dcti,Ncuan,normaliz,To,dUmax,dVmax);

W=1;

val_r=To;

e(:,:,1)=imdpcm(Lm(:,:,1),W,val_r(1));
e(:,:,2)=imdpcm(Lm(:,:,2),W,val_r(2));
e(:,:,3)=imdpcm(Lm(:,:,3),W,val_r(3));

N=round(0.9*[40 10 10]);

for i=1:3
    e1=e(:,:,i);
    me=mini(e1);
    Me=maxi(e1);

    extr_err(:,i)=[me Me]';

    QE(:,:,i)=round((N(i)-1)*(e1-me)/(Me-me));

end

masc=zeros(16,16);
masc(1:8,1:8)=ones(8,8);
masc=repmat(masc,16,16);
dct_contr(:,:,2)=masc.*dct_contr(:,:,2);
dct_contr(:,:,3)=masc.*dct_contr(:,:,3);

   N=round(0.9*[40 10 10]);
   for i=1:3
      qe(:,:,i)=(extr_err(2,i)-extr_err(1,i))*QE(:,:,i)/(N(i)-1)+extr_err(1,i);
      LumQ(:,:,i)=iimdpcm(qe(:,:,i),1,val_r(i));
   end

R=(1:255);R=R(:);

C=40000;
perfil_C=ones(255,1);

perfil_e=ones(255,1);

Lb = lcuan;
Li = tam;

Results=[];

codigazoA=[];
codigazoC1=[];
codigazoC2=[];

codigazoA_eob=[];
codigazoC1_eob=[];
codigazoC2_eob=[];

[h,alpha,beta,gamm]=constrains_respY(exponente(1),cero,tipo_alfa);
[hU,alphaU,betaU,gammU]=constrains_respU(exponente(2),cero);
[hV,alphaV,betaV,gammV]=constrains_respV(exponente(3),cero);

epsilon=epsilonv;

betasc =  zeros(256,255,3);
betascr= zeros(256,255,3);

fprintf('  Training all the blocks of the local non-linear domain \n');

for cap=1:capas
    fprintf('    Channel %d: ',cap);
    bloquecito=1;
    for i=1:Lb:Li
        fprintf('.');
        for j=1:Lb:Li

            aa=dct_contr(i:(i+(Lb-1)),j:(j+(Lb-1)),cap);
            azig = zigzag(aa);
            azigsc = azig(2:end);

            signo = azigsc<0;

            if nargin<13
                if cap==1
                    [h_a,r,GR,alfa2d,beta2d]=respue5(aa,alpha,beta,gamm,h,0);
                elseif cap==2
                    [h_a,r,GR,alfa2dU,beta2dU]=respue5(aa,alphaU,betaU,gammU,hU,0);
                else
                    [h_a,r,GR,alfa2dV,beta2dV]=respue5(aa,alphaV,betaV,gammV,hV,0);
                end
                r=zigzag(r);

                r = r(2:256).*(signo==0) - r(2:256).*(signo==1);

                RESP(bloquecito,:,cap)=r';

            else

                r=RESP(bloquecito,:,cap)';

            end

            [nsv,pesosSVM,bias] = irwls_pd_svr_nobias(R,r,K,C,perfil_C,epsilon(cap),perfil_e);
            betasc(bloquecito,:,cap)=pesosSVM';
            bloquecito=bloquecito+1;
        end
    end
    fprintf('\n');
end
fprintf('\n');

for cap=1:capas

    NumNiv(cap) = 2^bits(cap);

    betascr(:,:,cap) = round( NumNiv(cap) * betasc(:,:,cap)/MM)*MM/NumNiv(cap);

    rc=[];
    for fila=1:Li
        rc= [rc; (K*betascr(fila,:,cap)')'];
    end

    if cap==1

        codigonA(:,:,cap) = betascr(:,:,cap);
        codigonAint(:,:,cap)=codigonA*NumNiv(cap)/MM;

        for i=1:Lb^2
            [cod,ceros]=rle2(codigonAint(i,:,cap));

            codigazoA_eob=[codigazoA_eob cod ceros 1e8];
            codigazoA=[codigazoA cod ceros];
        end

    else

        codigonCcompleto(:,:,cap-1)=betascr(:,:,cap);
        codigonCintcompleto(:,:,cap-1)=codigonCcompleto(:,:,cap-1)*NumNiv(cap)/MM;

        for i=1:Lb^2

            bet = betascr(i,:,cap);
            bet=dezigzag([0 bet]);
            bet=bet(1:8,1:8);
            bet=zigzag(bet);
            codigonC(i,:,cap-1)=bet(2:end);
            codigonCint(i,:,cap-1)=codigonC(i,:,cap-1)*NumNiv(cap)/MM;

            [cod,ceros]=rle2(codigonCint(i,:,cap-1));

            if cap==2
                codigazoC1_eob=[codigazoC1_eob cod ceros 1e8];
                codigazoC1=[codigazoC1 cod ceros];
            else
                codigazoC2_eob=[codigazoC2_eob cod ceros 1e8];
                codigazoC2=[codigazoC2 cod ceros];
            end
        end
    end

end

repBloquesY=dct_ac_rows_to_patches(codigonAint);
repBloquesU=dct_ac_rows_to_patches(codigonCintcompleto(:,:,1));
repBloquesV=dct_ac_rows_to_patches(codigonCintcompleto(:,:,2));

ristra = [extr_err(:)' round(QE(:))' bits round(codigazoA) round(codigazoC1) round(codigazoC2)];

cd(directorio);
save_code(ristra,[fichero '.bin']);

zip(fichero,[fichero '.bin']);

delete([fichero '.bin']);

cd(you_are_here)
