% CONTRAS3 transforms to contrast local DCT amplitudes
% with Ncuan resolution in an YUV image.
%
% Ncuan resolution means blocks of size L_b = L/(2^Ncuan), where L is the image side size.
%
%
% CONTRAS3 allows three types of normalization:
%
% * Using the overall average image color (norm = 1)
% * Using the average color of each block (norm = 2)
% * Using color reference provided in To (in RGB NTSC) (norm = 3)
% 	For reference, the average color of the 19 analyzed images (in RGB NTSC
% 	uncalibrated) is To = [132 116 90] (which has a luminance of 118 cd/m2)
%
%
% The inverse is ICONTRAS3
%
% USE: [dctf,Lm]=contras3(dcti,Ncuan,Norm,To,DUmax,DVmax);
%

function [dctf,Lm]=contras3(s,Ncuan,norm,To,DUmax,DVmax)

   ss=s;
   tam=size(ss);

   if length(tam)==3
      capas=tam(3);
   else
      capas=1;
   end
   tam=tam(1);

   lados=[tam tam];

   dctf=zeros(tam,tam,capas);

   lcuan=tam/(2^Ncuan);

   posai=[tam tam]/2-round(tam/2);
   posbd=[tam tam]/2+round(tam/2);
   coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
   coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
   if coorcuanai(1)<1
        coorcuanai(1)=1;
   end
   if coorcuanai(2)<1
        coorcuanai(2)=1;
   end
   if coorcuanbd(1)>(2^Ncuan)
        coorcuanbd(1)=2^Ncuan;
   end
   if coorcuanai(2)>(2^Ncuan)
        coorcuanbd(2)=2^Ncuan;
   end

   Mrgb_yuv =[0.2990    0.5870    0.1140;-0.1726   -0.3388    0.5114;0.5114   -0.4282   -0.0832];
   Myuv_rgb = inv(Mrgb_yuv);

if norm==1
    DCT11=[0 0 0];
    Nbloques=0;
    for ii=coorcuanai(1):coorcuanbd(1)
        for jj=coorcuanai(2):coorcuanbd(2)

            iii=(ii-1)*lcuan+ceil(lcuan/2);
            jjj=(jj-1)*lcuan+ceil(lcuan/2);

            for i=1:capas
                s=ss(:,:,i);
                blo=sacasub(s,[iii jjj],[lcuan lcuan],0);
                DCT11(i)=DCT11(i)+blo(1,1);
            end
            Nbloques=Nbloques+1;

        end
    end
    DCT11=DCT11/Nbloques;
    if capas==3
       To_yuv=DCT11/lcuan;
       To=(Myuv_rgb*To_yuv')';
    else
       To=DCT11/lcuan;
       To_yuv=To;
    end
elseif norm==2
    To=0;
else
    if capas==3
       To_yuv=(Mrgb_yuv*To')';
    else
       To_yuv=To;
    end
end

factores_DCT_discreta=2*ones(lcuan,lcuan)/lcuan;
factores_DCT_discreta(1,:)=sqrt(2)*ones(1,lcuan)/lcuan;
factores_DCT_discreta(:,1)=sqrt(2)*ones(lcuan,1)/lcuan;
factores_DCT_discreta(1,1)=1/lcuan;

dctf=ss;

for i=1:capas

    s=ss(:,:,i);
    for ii=coorcuanai(1):coorcuanbd(1)
        for jj=coorcuanai(2):coorcuanbd(2)

            iii=(ii-1)*lcuan+ceil(lcuan/2);
            jjj=(jj-1)*lcuan+ceil(lcuan/2);
            blo=sacasub(s,[iii jjj],[lcuan lcuan],0);

            if i==1
                if norm==1
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=deltaT/To_yuv(1);
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                elseif norm==2
                    deltaT=factores_DCT_discreta.*blo;
                    To=deltaT(1,1);
                    contraste=deltaT/To;
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                else
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=deltaT/To_yuv(1);
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                end
            else
                if norm==1
                    if i==2
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);

                    else
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);

                    end
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=(2/YW)*deltaT/div;

                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                elseif norm==2
                    To_yuv=[0;0;0];
                    for lala=1:3
                        bobo=sacasub(ss(:,:,lala),[iii jjj],[lcuan lcuan],0);
                        To_yuv(lala)=bobo(1,1)/lcuan;
                    end
                    To=Myuv_rgb*To_yuv;
                    if i==2
                       W=To;
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);
                    else
                       W=To;
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);
                    end
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=(2/YW)*deltaT/div;
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                else
                    if i==2
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);
                    else
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);
                    end
                    deltaT=factores_DCT_discreta.*blo;
                    contraste=(2/YW)*deltaT/div;
                    Lm(ii,jj,i)=deltaT(1,1);
                    contraste(1,1)=0;
                end
            end
            dctf(:,:,i)=metesub(dctf(:,:,i),[iii jjj],contraste);
        end
    end
end

