% REPARTO_NIVELES returns the weight functions from a linear response model.
%
% Passing the result to respue_lin this scale DCTs in contrast for
% each channel, preparing them for the quantization. The result is
% equivalent to the inverse of a row of JPEG quantization matrices.
%
% This program allows you to generate multiple profiles sharing:
%
% Option = 1 of CSFs with scaling Kelly Mullen
%
% Option = 2 Ngan CSF for the achromatic channel and CSFs Kelly for chromatic
%
% Option = 3 low-pass filters with tunable frequency cut
%
% Option = 4 uniform Profile
%
% USE: [k1]=reparto_niveles(tam,fs,opcion,Fc);
%
%     tam = DCT block size
%     fs  = sampling frequency (cl/grad)
%     opcion = Profile
%     Fc = cut frequency (option 3)
%

function k=deal_levels(tam,fs,opcion,Fc,Pesos);

f=linspace(0,fs/2-(fs/2)/tam,tam);

if opcion==1

    factores_sobre_crom=[0.75 0.55];

    facfrecy=0.95;
    [IAFet,CSFy]=iafet(f,facfrecy,0,0,[0 0 0],0);

    facfrecrg=2.5;
    [iaf_rg,CSFrg]=iafrg(f,0,facfrecrg,[0 0 0]);
    facfrecyb=1.8;
    [iaf_yb,CSFyb]=iafyb(f,0,facfrecyb,[0 0 0]);

    CSFy=CSFy/maxi(CSFy);
    CSFrg=factores_sobre_crom(1)*CSFrg/maxi(CSFrg);
    CSFyb=factores_sobre_crom(2)*CSFyb/maxi(CSFyb);

    k=[CSFy;CSFyb;CSFrg];

elseif opcion==2

    factores_sobre_crom=[0.75 0.55];

    [CSFy,logciaf]=ngan(fs/2-(fs/2)/tam,tam);

    facfrecrg=2.5;
    [iaf_rg,CSFrg]=iafrg(f,0,facfrecrg,[0 0 0]);
    facfrecyb=1.8;
    [iaf_yb,CSFyb]=iafyb(f,0,facfrecyb,[0 0 0]);

    CSFy=CSFy/maxi(CSFy);
    CSFrg=factores_sobre_crom(1)*CSFrg/maxi(CSFrg);
    CSFyb=factores_sobre_crom(2)*CSFyb/maxi(CSFyb);

    k=[CSFy;CSFyb;CSFrg];

elseif opcion==3

    k=double([f<=Fc(1);f<=Fc(2);f<=Fc(3)]);

else

    k=ones(3,tam);

end

k(1,:)=Pesos(1)*k(1,:);
k(2,:)=Pesos(2)*k(2,:);
k(3,:)=Pesos(3)*k(3,:);

