% ICONTRAS3 is the inverse of CONTRAS3.
%
%
% CONTRAS3 allows three types of normalization:
%
% * Using the overall average image color (norm = 1)
% * Using the average color of each block (norm = 2)
% * Using color reference provided in To (in RGB NTSC) (norm = 3)
% 	For reference, the average color of the 19 analyzed images (in RGB NTSC
% 	uncalibrated) is To = [132 116 90] (which has a luminance of 118 cd/m2)
%
%
% The inverse is ICONTRAS3
%
%
% USE: [dctf]=icontras3(dcti,Lm,Ncuan,Norm,To,DUmax,DVmax);

function [dctf]=contras3(s,Lm,Ncuan,norm,To,DUmax,DVmax)

   ss=s;
   tam=size(ss);

   if length(tam)==3
      capas=tam(3);
   else
      capas=1;
   end
   tam=tam(1);

   lados=[tam tam];

   dctf=zeros(tam,tam,capas);

   lcuan=tam/(2^Ncuan);

   posai=[tam tam]/2-round(tam/2);
   posbd=[tam tam]/2+round(tam/2);
   coorcuanai=floor([(posai(1)-1)/lcuan+1 (posai(2)-1)/lcuan+1]);
   coorcuanbd=floor([(posbd(1)-1)/lcuan+1 (posbd(2)-1)/lcuan+1]);
   if coorcuanai(1)<1
        coorcuanai(1)=1;
   end
   if coorcuanai(2)<1
        coorcuanai(2)=1;
   end
   if coorcuanbd(1)>(2^Ncuan)
        coorcuanbd(1)=2^Ncuan;
   end
   if coorcuanai(2)>(2^Ncuan)
        coorcuanbd(2)=2^Ncuan;
   end

   Mrgb_yuv =[0.2990    0.5870    0.1140;-0.1726   -0.3388    0.5114;0.5114   -0.4282   -0.0832];
   Myuv_rgb = inv(Mrgb_yuv);

if norm==1

    if capas==3
       To_yuv=[mean(mean(Lm(:,:,1))) mean(mean(Lm(:,:,2))) mean(mean(Lm(:,:,3)))];
       To=(Myuv_rgb*To_yuv')';
    else
       To=mean(mean(Lm));
       To_yuv=To;
    end

elseif norm==2
    To=0;
else
    if capas==3
       To_yuv=(Mrgb_yuv*To')';
    else
       To_yuv=To;
    end
end

factores_DCT_discreta=2*ones(lcuan,lcuan)/lcuan;
factores_DCT_discreta(1,:)=sqrt(2)*ones(1,lcuan)/lcuan;
factores_DCT_discreta(:,1)=sqrt(2)*ones(lcuan,1)/lcuan;
factores_DCT_discreta(1,1)=1/lcuan;

dctf=ss;

for i=1:capas

    s=ss(:,:,i);
    for ii=coorcuanai(1):coorcuanbd(1)
        for jj=coorcuanai(2):coorcuanbd(2)

            iii=(ii-1)*lcuan+ceil(lcuan/2);
            jjj=(jj-1)*lcuan+ceil(lcuan/2);
            blo=sacasub(s,[iii jjj],[lcuan lcuan],0);

            if i==1
                if norm==1
                    contraste=blo;
                    deltaT=contraste*To_yuv(1);
                    blo=deltaT./factores_DCT_discreta;
                    blo(1,1)=Lm(ii,jj,i)./factores_DCT_discreta(1,1);
                elseif norm==2
                    contraste=blo;
                    To=Lm(ii,jj,i);
                    deltaT=contraste*To;
                    blo=deltaT./factores_DCT_discreta;
                    blo(1,1)=Lm(ii,jj,i)./factores_DCT_discreta(1,1);
                else
                    contraste=blo;
                    deltaT=contraste*To_yuv(1);
                    blo=deltaT./factores_DCT_discreta;
                    blo(1,1)=Lm(ii,jj,i)./factores_DCT_discreta(1,1);
                end
            else
                if norm==1
                    if i==2
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);

                    else
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);

                    end
                    contraste=blo;
                    deltaT=(YW/2)*div*contraste;
                    blo=deltaT./factores_DCT_discreta;
                    blo(1,1)=Lm(ii,jj,i)./factores_DCT_discreta(1,1);
                elseif norm==2
                    To_yuv=[Lm(ii,jj,1);Lm(ii,jj,2);Lm(ii,jj,3)];
                    To=Myuv_rgb*To_yuv;
                    if i==2
                       W=To;
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);
                    else
                       W=To;
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);
                    end
                    contraste=blo;
                    deltaT=(YW/2)*div*contraste;
                    blo=deltaT./factores_DCT_discreta;
                    blo(1,1)=Lm(ii,jj,i)./factores_DCT_discreta(1,1);
                else
                    if i==2
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;DUmax;0]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(2);
                    else
                       W=To';
                       YW=To_yuv(1);
                       e1=(Myuv_rgb*[0;0;DVmax]+W)/YW;
                       e2=2*W/YW-e1;
                       div=Mrgb_yuv*(e1-e2);
                       div=div(3);
                    end
                    contraste=blo;
                    deltaT=(YW/2)*div*contraste;
                    blo=deltaT./factores_DCT_discreta;
                    blo(1,1)=Lm(ii,jj,i)./factores_DCT_discreta(1,1);
                end
            end
            dctf(:,:,i)=metesub(dctf(:,:,i),[iii jjj],blo);
        end
    end
end
