% Given a value for a distortion (RMSE o SSIM) it iterates to reach
% an image with this distortion.
%
% It encodes the image, gets a  "code" and decodes the image using
% the JPEG algorithm.
%
% It computes the distortion and reset the uCP parameter if necessary,
% and starts again.
% It gives back an struct with the encoder results and the decoded image.

function [Results_JPEG] = distortion_jpeg(Im,normaliz,To,dUmax,dVmax,uCPv,variable,parametro,directorio,fichero,niveles,bitmax)
Results_JPEG = [];

if(nargin < 12 && niveles == 1)
    bitmax = 1;
end

pesoY = 0.7;
pesoU = 0.15;
pesoV = 0.15;
for i = 1:length(parametro)

    uCP = [0 0 0];

    iter = 0;

    porcentaje = 1;

    if(isequal(variable,'SSIM'))
        sentido = 1;
    else
        sentido = 0;
    end

    l_min = 0;
    l_max = 100000;

    switch(variable)
        case 'RMSE', parametro_actual = 0; umbral = 0.01;
        case 'SSIM', parametro_actual = 0; umbral = 0.001;
        case 'MPE lineal', parametro_actual = 100; umbral = 0.01;
        case 'MPE no lineal', parametro_actual = 100; umbral = 0.01;
        case 'MPE no lineal JMLR', parametro_actual = 100; umbral = 0.1;
        case 'SCIELab', parametro_actual = 0; umbral = 0.01;
    end

    no_results = 0;
    disp(['Ajustando el uCP para hallar un ' variable ' de:   ' num2str(parametro(i))]);
    while((abs(parametro(i) - parametro_actual) > umbral) && (iter < 30))
        iter = iter + 1;

        if(iter ~= 1 && iter ~= 2)
            if(isequal(variable,'SSIM'))
                [uCP2, l_min, l_max, porcentaje, sentido] = fitting_uCP_SSIM(parametro(i), parametro_actual, uCP(1), l_min, l_max, porcentaje, sentido);
            else
                [uCP2, l_min, l_max, porcentaje, sentido] = fitting_uCP(parametro(i), parametro_actual, uCP(1), l_min, l_max, porcentaje, sentido);
            end

            uCP = uCP2 .* [1 1 1];
        end

        jpeg91_c_encoder(Im,normaliz,To,dUmax,dVmax,uCP,0,directorio,[fichero '_' num2str(i)],niveles,bitmax);

        ImR_jpeg = jpeg91_c_decoder(normaliz,To,dUmax,dVmax,directorio,[fichero '_' num2str(i)],niveles,bitmax);

        mse = zeros(3,1);
        esesim = zeros(3,1);
        MPEl = zeros(3,1);
        MPEnl = zeros(3,1);
        MPEnlJMLR = zeros(3,1);
        SCIELab = 0;

        Im_yuv = my_rgb2yuv(double(Im));
        ImR_JPEG_yuv = my_rgb2yuv(ImR_jpeg);

        for capa = 1:3
            switch(variable)
                case 'RMSE',
                    [mse(capa),SNRi] = emse2(double(Im_yuv(:,:,capa)),ImR_JPEG_yuv(:,:,capa));
                case 'SSIM',
                    esesim(capa) = ssim_index(double(Im_yuv(:,:,capa)),ImR_JPEG_yuv(:,:,capa));
                case 'MPE lineal',
                    [MSE,MPEl(capa),dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(double(Im_yuv(:,:,capa)),ImR_JPEG_yuv(:,:,capa),2,[3 mean(mean(Im_yuv(:,:,capa)))],2,2);
                case 'MPE no lineal',
                    [MSE,MPEnl(capa),dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im_yuv(:,:,capa)),ImR_JPEG_yuv(:,:,capa),[3 mean(mean(Im_yuv(:,:,capa)))],2,1,2,2);
                case 'MPE no lineal JMLR',
                    [MSE,MPEnlJMLR(capa),dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im_yuv(:,:,capa)),ImR_JPEG_yuv(:,:,capa),[3 mean(mean(Im_yuv(:,:,capa)))],1,0.5,2,2);
            end
        end
        if(isequal(variable,'SCIELab'))
            SCIELab = calculo_scielab(Im,ImR_jpeg);
        end
        switch(variable)
            case 'RMSE',
                mse = pesoY * mse(1) + pesoU * mse(2) + pesoV * mse(3);
                parametro_actual = sqrt(mse);
            case 'SSIM',
                esesim = pesoY * esesim(1) + pesoU * esesim(2) + pesoV * esesim(3);
                parametro_actual = esesim;
            case 'MPE lineal',
                parametro_actual = pesoY * MPEl(1) + pesoU * MPEl(2) + pesoV * MPEl(3);
            case 'MPE no lineal',
                parametro_actual = pesoY * MPEnl(1) + pesoU * MPEnl(2) + pesoV * MPEnl(3);
            case 'MPE no lineal JMLR',
                parametro_actual = pesoY * MPEnlJMLR(1) + pesoU * MPEnlJMLR(2) + pesoV * MPEnlJMLR(3);
            case 'SCIELab'
                parametro_actual = SCIELab;
        end
        disp(['Iteracion:  ' num2str(iter) '   ' variable ':  ' num2str(parametro_actual) '   uCP:  ' num2str(uCP(1))]);

        if(~isequal(variable,'SSIM') && (iter == 1) && (parametro(i) - parametro_actual > 0) && (uCP(1) == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP.');
            break;

        elseif(isequal(variable,'SSIM') && (iter == 1) && (parametro(i) - parametro_actual < 0) && (uCP(1) == 0))
            no_results = 1;
            disp('No se puede ajustar el uCP.');
            break;

        elseif((iter == 1) && (abs(parametro(i) - parametro_actual) <= umbral) && (uCP(1) == 0))

            uCP = uCPv;

            parametro_actual = 0;

        elseif(iter == 1)
            uCP = uCPv;
        end
    end
    if(no_results == 0)

        cd(directorio)
        unzip([fichero '_' num2str(i)]);
        ristra = read_code([fichero '_' num2str(i) '.bin']);

        delete([fichero '_' num2str(i) '.bin']);

        D = dir([directorio fichero '_' num2str(i) '.zip']);
        tamanyo = 8 * D.bytes;

        entropia = tamanyo / prod([size(ImR_jpeg,1) size(ImR_jpeg,2)]);

        Results_JPEG(i).Normalizacion = normaliz;
        Results_JPEG(i).To = To;
        Results_JPEG(i).dUmax = dUmax;
        Results_JPEG(i).dVmax = dVmax;
        Results_JPEG(i).Niveles = niveles;
        Results_JPEG(i).Entropy = entropia;
        Results_JPEG(i).uCP = uCP;
        Results_JPEG(i).Image = ImR_jpeg;
        Results_JPEG(i).ristra = ristra;
        switch(variable)
            case 'RMSE',
                Results_JPEG(i).RMSE = parametro_actual;
            case 'SSIM',
                Results_JPEG(i).SSIM = parametro_actual;
            case 'MPE lineal',
                Results_JPEG(i).MPE_lineal = parametro_actual;
            case 'MPE no lineal',
                Results_JPEG(i).MPE_no_lineal = parametro_actual;
            case 'MPE no lineal JMLR',
                Results_JPEG(i).MPE_no_lineal_JMLR = parametro_actual;
            case 'SCIELab'
                Resutls_JPEG(i).SCIELab = parametro_actual;
        end
    end
end
