
% Given a value for a distortion (RMSE o SSIM) it iterates to reach
% an image with this distortion.
% It encodes the image, gets a code and decodes the image using
% the nl+svr algorithm.
% It computes the distortion and reset the epsilon parameter if necessary,
% and starts again.
% It gives back an struct with the encoder results and the decoded image.
%
% NOTE THAT: The distortion values vector has to be sorted descent for all the
% distortions but for the SSIM, which will sort in ascent.

function [Results_NL_SVR] = distortion_nl_svr(Im,normaliz,To,dUmax,dVmax,epsilonv,K,exponente,tipo_alfa,variable,parametro,directorio,fichero)
Results_NL_SVR = [];

pesoY = 0.7;
pesoU = 0.15;
pesoV = 0.15;

ini_bits = 2;
fin_bits = 15;
for i = 1:length(parametro)

    ImR_nl_svr = [];
    disp(['Ajustando el epsilon para hallar el ' variable ' de:   ' num2str(parametro(i))]);
    for b = ini_bits:0.5:fin_bits
        switch(variable)
            case 'RMSE', parametro_actual = 0; umbral = 0.01;
            case 'SSIM', parametro_actual = 0; umbral = 0.001;
            case 'MPE lineal', parametro_actual = 100; umbral = 0.01;
            case 'MPE no lineal', parametro_actual = 100; umbral = 0.01;
            case 'MPE no lineal JMLR', parametro_actual = 100; umbral = 0.1;
            case 'SCIELab', parametro_actual = 0; umbral = 0.01;
        end

        epsilon = [0 0 0];

        bits = b .* [1 1 1];

        iter = 0;

        porcentaje = 1;

        if(isequal(variable,'SSIM'))
            sentido = 0;
        else
            sentido = 1;
        end

        l_min = 0;
        l_max = 100000;

        no_results = 0;
        while((abs(parametro(i) - parametro_actual) > umbral) & (iter < 30))
            iter = iter + 1;

            if(iter ~= 1 && iter ~= 2)
                if(isequal(variable,'SSIM'))
                    [epsilon2, l_min, l_max, porcentaje, sentido] = fitting_epsilon_SSIM(parametro(i), parametro_actual, epsilon(1), l_min, l_max, porcentaje, sentido);
                else
                    [epsilon2, l_min, l_max, porcentaje, sentido] = fitting_epsilon(parametro(i), parametro_actual, epsilon(1), l_min, l_max, porcentaje, sentido);
                end
                epsilon = epsilon2 .* [1 1 1];
            end

            [encoder_nl_svr] = svr_nl_c_encoder_cs(Im,normaliz,To,dUmax,dVmax,bits,epsilon,K,exponente,tipo_alfa,directorio,[fichero '_' num2str(i)]);
            close all

            [ImR_nl_svr] = svr_nl_c_decoder_cs(normaliz,To,dUmax,dVmax,K,exponente,tipo_alfa,directorio,[fichero '_' num2str(i)]);

            mse = zeros(3,1);
            esesim = zeros(3,1);
            MPEl = zeros(3,1);
            MPEnl = zeros(3,1);
            MPEnlJMLR = zeros(3,1);
            SCIELab = 0;

            Im_yuv = my_rgb2yuv(double(Im));
            ImR_NL_SVR_yuv = my_rgb2yuv(ImR_nl_svr);

            for capa = 1:3
                switch(variable)
                    case 'RMSE',
                        [mse(capa),SNRi] = emse2(double(Im_yuv(:,:,capa)),ImR_NL_SVR_yuv(:,:,capa));
                    case 'SSIM',
                        esesim(capa) = ssim_index(double(Im_yuv(:,:,capa)),ImR_NL_SVR_yuv(:,:,capa));
                    case 'MPE lineal',
                        [MSE,MPEl(capa),dx0,MPEx0,MPEoo0,MPEexpesp0] = disdis_lineal(double(Im_yuv(:,:,capa)),ImR_NL_SVR_yuv(:,:,capa),2,[3 mean(mean(Im_yuv(:,:,capa)))],2,2);
                    case 'MPE no lineal',
                        [MSE,MPEnl(capa),dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im_yuv(:,:,capa)),ImR_NL_SVR_yuv(:,:,capa),[3 mean(mean(Im_yuv(:,:,capa)))],2,1,2,2);
                    case 'MPE no lineal JMLR',
                        [MSE,MPEnlJMLR(capa),dx,MPEx,MPEoo,MPEexpesp] = disdis7(double(Im_yuv(:,:,capa)),ImR_NL_SVR_yuv(:,:,capa),[3 mean(mean(Im_yuv(:,:,capa)))],1,0.5,2,2);
                end
            end
            if(isequal(variable,'SCIELab'))
                SCIELab = calculo_scielab(Im,ImR_nl_svr);
            end
            switch(variable)
                case 'RMSE',
                    mse = pesoY * mse(1) + pesoU * mse(2) + pesoV * mse(3);
                    parametro_actual = sqrt(mse);
                case 'SSIM',
                    esesim = pesoY * esesim(1) + pesoU * esesim(2) + pesoV * esesim(3);
                    parametro_actual = esesim;
                case 'MPE lineal',
                    parametro_actual = pesoY * MPEl(1) + pesoU * MPEl(2) + pesoV * MPEl(3);
                case 'MPE no lineal',
                    parametro_actual = pesoY * MPEnl(1) + pesoU * MPEnl(2) + pesoV * MPEnl(3);
                case 'MPE no lineal JMLR',
                    parametro_actual = pesoY * MPEnlJMLR(1) + pesoU * MPEnlJMLR(2) + pesoV * MPEnlJMLR(3);
                case 'SCIELab'
                    parametro_actual = SCIELab;
            end
            disp(['Iteracion:  ' num2str(iter) '   ' variable ':  ' num2str(parametro_actual) '   bits:  ' num2str(bits(1)) '   epsilon:  ' num2str(epsilon(1))]);

            if(~isequal(variable,'SSIM') && (iter == 1) && (parametro(i) - parametro_actual < 0) && (epsilon(1) == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits.');
                break;

            elseif(isequal(variable,'SSIM') && (iter == 1) && (parametro(i) - parametro_actual > 0) && (epsilon(1) == 0))
                no_results = 1;
                disp('No se puede ajustar el epsilon para este número de bits.');
                break;

            elseif((iter == 1) && (abs(parametro(i) - parametro_actual) <= umbral) && (epsilon(1) == 0))

                epsilon = epsilonv;

                parametro_actual = 0;

            elseif(iter == 1)
                epsilon = epsilonv;
            end
        end
        if(no_results == 0)

            cd(directorio)
            unzip([fichero '_' num2str(i)]);
            ristra = read_code([fichero '_' num2str(i) '.bin']);

            delete([fichero '_' num2str(i) '.bin']);

            D = dir([directorio fichero '_' num2str(i) '.zip']);
            tamanyo = 8 * D.bytes;

            entropia = tamanyo / size(ImR_nl_svr,1)^2;

            Results_NL_SVR(i).Normalizacion = normaliz;
            Results_NL_SVR(i).To = To;
            Results_NL_SVR(i).dUmax = dUmax;
            Results_NL_SVR(i).dVmax = dVmax;
            Results_NL_SVR(i).Bits = bits;
            Results_NL_SVR(i).Epsilon = epsilon;
            Results_NL_SVR(i).Kernel = K;
            Results_NL_SVR(i).Exponente = exponente;
            Results_NL_SVR(i).Tipo_alfa = tipo_alfa;
            Results_NL_SVR(i).Entropy = entropia;
            Results_NL_SVR(i).Image = ImR_nl_svr;
            Results_NL_SVR(i).ristra = ristra;
            switch(variable)
                case 'RMSE',
                    Results_NL_SVR(i).RMSE = parametro_actual;
                case 'SSIM',
                    Results_NL_SVR(i).SSIM = parametro_actual;
                case 'MPE lineal',
                    Results_NL_SVR(i).MPE_lineal = parametro_actual;
                case 'MPE no lineal',
                    Results_NL_SVR(i).MPE_no_lineal = parametro_actual;
                case 'MPE no lineal JMLR',
                    Results_NL_SVR(i).MPE_no_lineal_JMLR = parametro_actual;
                case 'SCIELab'
                    Results_NL_SVR(i).SCIELab = SCIELab;
            end

            ini_bits = b;

            break;
        end
    end
end
