% function [rest]=regu_operator(Imobservada,b,lambdas,N,M,F,criterio_lambda,pinta);
%
%   Given an observed degraded image (given by 'K' and additive noise)
%   it regularizes using the a penalization operator F and reguilarization parameters
%   'lambdas'.
%   N is the block size and M is the block size plus the boundary
%   (in order to avoid boundary effects in the reconstruction.
%   It gives back also the image blocks in order to perform the overlapping.
%
%      i.e.
%      N=16;
%      M=44;
%      Nlambdas=70;
%      lambdas=logspace(-2,2,Nlambdas);
%      operador=fft2(filt2dsegder,M,M);
%
%      figure,[Irest]=regu_operator(BlurredNoisy,PSF,lambdas,N,M,operador,criterio_lambda,pinta);
%

function [restaurada,objetoparaoverlapping]=regu_operator(Imobservada,b,lambdas,N,M,F,criterio_lambda,pinta,varargin);

sinruido=wiener2(Imobservada,[3 3]);

noiseest = mean(var(sinruido-Imobservada));

normaruido = sqrt(noiseest)*M;

[fil,col]=size(Imobservada);

restaurada=rand([fil,col]);

nver = fil/N;
nhor = col/N;

objetoparaoverlapping = rand(nver*M,nhor*M);

a=(M-N)/2;

Im2=ampliaconborde(Imobservada,a);

nlamb=length(lambdas);

B=freqz2(b,M,M);

B=ifftshift(B);

contadorfilas=0;

if (nargin==8)

   for i=N/2:N:fil

      contadorfilas = contadorfilas+1;
      contadorcolumnas = 0;

      for j=N/2:N:col

         contadorcolumnas = contadorcolumnas + 1;

         observado=Im2(i+a-M/2+1:i+a+M/2,j+a-M/2+1:j+a+M/2);

         [sol]=restaura_block(observado,B,F,M,N,lambdas,criterio_lambda);

         restaurada(i-N/2+1:i+N/2,j-N/2+1:j+N/2)=sol(a+1:a+N,a+1:a+N);

         if (pinta==1)
            imshow(restaurada,[0 256],'notruesize'),pause(0.2);
         end

         objetoparaoverlapping((contadorfilas-1)*M+1:contadorfilas*M,(contadorcolumnas-1)*M+1:contadorcolumnas*M) = sol;

      end
   end

   restaurada = do_overlap(objetoparaoverlapping,M,N,8);

else
   centro = varargin{1};

   i = centro(1);
   j=centro(2);

   observado=Im2(i+a-M/2+1:i+a+M/2,j+a-M/2+1:j+a+M/2);

   [sol]=restaura_block(observado,B,F,M,N,lambdas,criterio_lambda);

   restaurada = sol;
end

