% It 'cleans' an image using bayesian formulation with laplacian prior
% as explained in Simoncelli '99.
% "Bayesian denoising of visual images in the wavelet domain."
%
% USE: [IMM,S,P]=Soft_B_denois_4_numeric(Im2,Var)
%
% INPUTS
% Im2 = noisy image;
% Var = noise variance
%
% OUTPUT
% IMM es la imagen limpiada;
% S and P are the estimated values that define the prior distributions
% for each wavelet scale

function [IMM,S,P]=Soft_B_denois_4_numeric(Im2,Var)

S=[]
ImDeg = Im2;
desviacion=sqrt(Var);
NW=4;
[pyrDeg,indDeg]=buildwpyr(double(ImDeg),NW);

orientaciones=cell(3,1);

orientaciones{1}='H';
orientaciones{2}='V';
orientaciones{3}='D';

estimado=zeros(size(pyrDeg));

cont=1;

for escala=1:NW
    escala
    for ori=1:3
        orientacion=orientaciones{ori}

        switch lower(orientacion)
            case 'h'
                sub=1;
            case 'v'
                sub=2;
            case 'd'
                sub=3;
        end

        [histograma,valores,subbanda,subbandacrop]=marginal_2(Im2,escala,orientacion,256);
        QQ=1
        for s=logspace(-1,2,25)
            for p=linspace(0.35,1.45,25)
                l=likelihooddegradedaB([s p],subbanda,Var);
                MM(1,QQ)=s;
                MM(2,QQ)=p;
                MM(3,QQ)=l;
                escala
                QQ=QQ+1
            end
        end
        [MN,IN]=min(MM(3,:));
        S(escala,ori)=MM(1,IN);
        P(escala,ori)=MM(2,IN);

        pyrsubDeg = wpyrBand(pyrDeg, indDeg, escala, sub);
        pyrsubDeg = pyrsubDeg(:);

        lookup = [];

        pasox=0.5;
        rango = [round(min(pyrsubDeg))-30:pasox:round(max(pyrsubDeg))+30];
        num2=[];den2=[];
        for valor=rango
            num = sum(pasox.*blsnum(rango,valor,S(escala,ori),P(escala,ori),desviacion));
            denom = sum(pasox.*blsdenom(rango,valor,S(escala,ori),P(escala,ori),desviacion));
            num2=[num2 num];
            den2=[den2 denom];
            lookup = [lookup num/denom];
        end

        for indice=1:length(pyrsubDeg)
            estimado(cont) = interp1(rango,lookup,pyrsubDeg(indice));
            cont=cont+1;
        end

    end
end

estimado(cont:end) = pyrDeg(cont:end);

IMM=reconWpyr(estimado,indDeg);

