% It 'cleans' an image using bayesian formulation with laplacian prior
% as explained in Simoncelli '99.
% "Bayesian denoising of visual images in the wavelet domain."
%
% USE: [IMM,S,P]=Soft_B_denois_steerable(Im2,Var)
%
% INPUTS
% Im2 = noisy image;
% Var = noise variance
%
% OUTPUT
% IMM es la imagen limpiada;
% S and P are the estimated values that define the prior distributions
% for each wavelet scale

function [IMM,S,P]=Soft_B_denois_steerable(Im2,Var)
ImDeg = Im2;
desviacion=sqrt(Var);

NW=4;
[pyrDeg,indDeg]=buildsfpyr(double(ImDeg),4,7);
pyrEst=zeros(size(pyrDeg));

load indices_epsilon_4esc_8ori
indices=indices*sqrt(Var)/sqrt(400);
for escala=1:NW
    for ori=1:8
        [escala ori]

        PP=[];
        A=[];
        for i=1:2^(9-escala)
            for j=1:2^(9-escala)
                pp=pos_s_pyr([escala ori i j],indDeg);
                A(i,j)=pyrDeg(pp);
                PP(i,j)=pp;
            end
        end
        subbanda=A;

        QQ=1;
        MM=zeros(3,15*15);
        for s=logspace(-1.5,2.5,25)
            for p=linspace(0.35,2,20)
                l=likelihooddegradedaB([s p],subbanda,indices(PP(1,1))^2);
                MM(1,QQ)=s;
                MM(2,QQ)=p;
                MM(3,QQ)=l;
                QQ=QQ+1;
            end
        end
        [MN,IN]=min(MM(3,:));
        S(escala,ori)=MM(1,IN);
        P(escala,ori)=MM(2,IN);

        pyrsubDeg = subbanda(:);
        lookup = [];

        pasox=0.5;
        rango = [round(min(pyrsubDeg))-30:pasox:round(max(pyrsubDeg))+30];
        num2=[];den2=[];
        for valor=rango
            num = sum(pasox.*blsnum(rango,valor,S(escala,ori),P(escala,ori),desviacion));
            denom = sum(pasox.*blsdenom(rango,valor,S(escala,ori),P(escala,ori),desviacion));
            num2=[num2 num];
            den2=[den2 denom];
            lookup = [lookup num/denom];
        end

        estimado=zeros(size(pyrsubDeg));
        for indice=1:length(pyrsubDeg)
            estimado(indice) = interp1(rango,lookup,pyrsubDeg(indice));
        end
        BB=reshape(estimado,2^(9-escala),2^(9-escala));
        pyrEst(PP)=BB;
    end
end
pyrEst(length(pyrDeg)-32*32+1:length(pyrDeg)) = pyrDeg(length(pyrDeg)-32*32+1:length(pyrDeg));
IMM=reconsfpyr(pyrEst,indDeg);
