function KeCoDe_coder_tools_demo_color

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                                  %
%%  KeCoDe  Demo 2:                 %
%%  Encoding-Decoding Color Images  %
%%                                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Read and execute this demo to learn how to encode-decode 256*256 color images
%  using KeCoDeCoderTools
%
%     This demo compares 2 algorithms:
%         - JPEG (with quantization matrices based on Mullen 85 CSFs)
%                linear human visual system model
%         - Gutierrez11:
%                general non-linear human visual system model (including
%                non-linearities in the chromatic channels) + SVR
%
%     It generates 2 files of the same size at the output folder provided
%     by the user and shows the results of the decoded images along with three
%     distortion measures RMSE, SSIM and S-CIELAB.
%
%     Target entropy is 1 bits/pix
%
%     This demo is illustrative but slow: it takes about 11 min to complete.
%     It is slow due to the loop to obtain the required entropy with every algorithm.
%
%    USE:  KeCoDe_coder_tools_demo_color
%
%    See details in:
%    [Gutierrez11] J. Gutierrez, G. Camps-Valls, M.J. Luque and J. Malo
%             A Color Contrast Definition for Perceptually based Color Image Coding
%             Recent Patents on Signal Processing. In press (2011)
%

% LOAD IMAGE

Im=imread('images/edificio1_256.tif');

% ENCODE THE IMAGE (KeCoDe_encoder_color.m)
%
% The loop below runs over the available algorithms (see help of
% KeCoDe_encoder_achrom.m) to generate the encoded version of the image (at
% the specified target entropy) in different files.

Target_entropy=1;    % Desired entropy (in bits/pix)

file_name='demo2';   % Note that algorithm identification will be added to this file name

% Output folder
you_are_here=pwd;
output_folder=which('KeCoDe_coder_tools_demo_achrom');
output_folder=output_folder(1:end-32);
cd(output_folder);
cd('tmp');
output_folder=pwd;
barra=output_folder(end-3);
output_folder=[output_folder,barra];
cd(you_are_here);
warning off

Num_iterat=10;       % Number of iterations used to achieve the desired file size
                     % (if this number is reduced to 1, a too crude
                     % approximation to the target entropy is obtained

for i=1:2
    Results = KeCoDe_encoder_color(Im,i,output_folder,file_name,Target_entropy,Num_iterat);
    eval(['Results',num2str(i),'=Results;'])
    clear Results
end

% DECODE THE FILES (KeCoDe_decoder_color.m)
%
% The loop below runs over the files generated by the previous loop
% available algorithms to decode them and obtain the decompressed
% images.

names=struct('name',[]);                 % Algorithm identifiers given by KeCoDe_encoder_color.m
names(1).name='_jpeg91_c_entropy_1';
names(2).name='_gutierrez09_c_entropy_1';

for i=1:2
    file=[file_name,names(i).name];
    im=KeCoDe_decoder_color(output_folder,file);
    eval(['Im',num2str(i),'=im;']);
    clear im
end

figure,subplot(131),imshow(Im),title('Original.  Entropy=24 bpp')
       subplot(132),
       imshow(Results1.Image),title(['JPEG-like quantiz.  Entropy=',num2str(Results1.Entropy),' bpp'])
       [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Results1.Image,[0.7 0.1 0.2]);       
       % xlabel(char(['  RMSE=',num2str(round(10*Results1.RMSE)/10) ]))
       xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})
       subplot(133),
       imshow(Results2.Image),title(['Gutierrez 09.  Entropy=',num2str(Results2.Entropy),' bpp'])
       [rmse,SSIM,d_l,d_DCT] = computing_distortions(Im,Results2.Image,[0.7 0.1 0.2]);       
       % xlabel(char(['  RMSE=',num2str(round(10*Results1.RMSE)/10) ]))
       xlabel({['MSE^{1/2} = ',num2str(round(rmse*10)/10),'  SSIM = ',num2str(round(SSIM*100)/100)],['d_{DCT} = ',num2str(round(d_DCT*10)/10)]})
im_orig = Im;       
im_jpeg = Results1.Image;
im_dn = Results2.Image;
save('/media/disk/vista/shit','im_orig','im_jpeg','im_dn')
cd(you_are_here)

